#!/usr/bin/perl

use strict;

my ($deltafile)=@ARGV; # cds can be replaced by utr's if available; TO BE IMPLEMENTED LATER
                                   # $deltafile = name of delta file generated by promer
                                   # if -c option is specified, delta file represents the file generated from
                                   # the delta file by using the show-cords program

use Getopt::Long;

my ($help,$pathmum,$outfilename,$verb,$flankreg,$coordsfile,$cds1,$cds2);

GetOptions( "h"    => \$help,
	    "p:s"  => \$pathmum,
            "o:s"  => \$outfilename,
            "v"    => \$verb,
	    "c"    => \$coordsfile,
	    "d:i"  => \$flankreg,
	    "r:s"  => \$cds1,
	    "q:s"  => \$cds2
            );

my $usage = <<'&EOT&';

USAGE : OrthoDetect.pl  <mumer output file>  [-r <ref CDS|UTR coords file>] [-q <query CDS|UTR coords file>] [options]

# mummer output is supposed to be between 2 multi fasta files

Options :
  -r <cds1> : gene-finding predictions in reference file
  -q <cds2> : gene-finding predictions in query file
  -p <path> : path to the show-coords script if not in current directory; 
  -o <file name> : the name of the output file
  -d <flanking region> : length of flanking region to consider (default=1000bp);
  -h : display this help;
  -c : if mummer output is in the format obtained after running show-coords script
  -v : verbosely list files processed;
 
  cds (or utr) coordinates files need to be in gff format. check the gff format for its standard

&EOT&

# you change here to have it similar to trainglimmerm where i detect the path of the running directory; still show-coords may be in a different location

if ($help)  {  die $usage."\n"; }

die $usage if (@ARGV<1);

my $allowedovlp=200;

if(!$flankreg) { $flankreg=1000;} # set default flanking region to 1000bp

### generate the coords files if not in input
my $refquecoords=$deltafile;
if(!$coordsfile) {
    $refquecoords.=".coords";
    if(!$pathmum) { 
	system("show-coords -c -l -r $deltafile > $refquecoords");
    }
    else {
	system("$pathmum\\show-coords -c -l -r $deltafile > $refquecoords");
    }
}

# read positions of the indices of interest in the querry and reference files
my ($ind_s1,$ind_e1,$ind_s2,$ind_e2,$ind_pidy,$ind_psim,$ind_lenref,$ind_lenque)=read_ind_positions($refquecoords);

# MAIN 


# :: Step 1 ::

### read all coordinates

# reference genome data
my $rcontigno=0;  # global variable should not be altered; stores no. of reference contigs
my @rcoordno;     # no. of records in reference contig 
my @rlcontig;     # reference contig lengths
my %rcontig;      # reference contig names
my @rcoord;       # rcoord = (value, start|end, +|-, alignment|gene, qcoordlink-contigno, qcoordlink-coordno, color, PSIM)

# query genome data
my $qcontigno=0; # global variable should not be altered; stores no. of query contigs
my @qcoordno;    # no. of records in query contig 
my @qlcontig;    # query contig lengths
my %qcontig;     # query contig names
my @qcoord;      # qcoord = (0:value, 1:start|end, 2:+|-, 3:alignment|gene, 4:rcoordlink-contigno, 5:rcoordlink-coordno, 6:color, 7:psim)

read_coords($refquecoords,$cds1,$cds2,$verb);


# :: Step 2 ::

### sort cordinates

# sort @{$rcoord[$i]} = > order stored in @{$srcoord[$i]}
my @srcoord;
sort_rcoord();

# sort @{$qcoord[$i]} = > order stored in @{$sqcoord[$i]}
my @sqcoord;
sort_qcoord();


# :: Step 3 ::

### determine non-conflicting regions; conflicting regions are marked with the color -1

my $bigcolor=parse_regions($verb);


# :: Step 4 ::

### here print non-conflicting regions

if($outfilename) { open(O,">$outfilename"); select O;}

print "    StartR       EndR |     StartQ       EndQ |  RACovLen  RGeneLen  NoEx NoGen       RCodCovLen(%) |  QACovLen  QGeneLen  NoEx NoGen       QCodCovLen(%) |   Conflict   | Tags\n";
print "====================================================================================================================================================================\n";

# R/QACovLen = Length of reference/query alignments which cover region
# R/QCodCovLen = coding length covered by alignment (and % from all coding length)


my $conflict="      -      ";
print_regions($conflict);

if($verb) { print STDERR "Done nonconflicting regions. Processing conflicting ones! rcontigno=$rcontigno qcontigno=$qcontigno\n";}
# :: Step 5 ::

### analyze conflicting regions 

# - "paint" overlapping conflicts with big color by choosing the longest one; these will be eliminated at next step
paint_overlap_conflicts($bigcolor+1);

if($verb) {
    print STDERR "Sorted Rcoords:\n";
    for(my $i=1;$i<=$rcontigno;$i++) {
	for(my $j=0;$j<=$rcoordno[$i];$j++) {
	    print STDERR "$i $j ",$srcoord[$i][$j]," ",$rcoord[$i][$srcoord[$i][$j]][0]," ",$rcoord[$i][$srcoord[$i][$j]][1]," ",$rcoord[$i][$srcoord[$i][$j]][2]," ",$rcoord[$i][$srcoord[$i][$j]][3]," ",$rcoord[$i][$srcoord[$i][$j]][4]," ",$rcoord[$i][$srcoord[$i][$j]][5]," ",$rcoord[$i][$srcoord[$i][$j]][6],"\n";
	}
    }
    print STDERR "Sorted Qcoords:\n";
    for(my $i=1;$i<=$qcontigno;$i++) {
	for(my $j=0;$j<=$qcoordno[$i];$j++) {
	    print STDERR "$i $j ",$sqcoord[$i][$j]," ",$qcoord[$i][$sqcoord[$i][$j]][0]," ",$qcoord[$i][$sqcoord[$i][$j]][1]," ",$qcoord[$i][$sqcoord[$i][$j]][2]," ",$qcoord[$i][$sqcoord[$i][$j]][3]," ",$qcoord[$i][$sqcoord[$i][$j]][4]," ",$qcoord[$i][$sqcoord[$i][$j]][5]," ",$qcoord[$i][$sqcoord[$i][$j]][6],"\n";
	}
    }
}

if($verb) { print STDERR "Overlaps resolved with color ",$bigcolor+1,".\n";}

# - delete nonconflicting regions; 
delete_ok_contigs();

if($verb) { print STDERR "Delete complete.\n";}

# - resort the rcoord and qcoord arrays
@srcoord=();
sort_rcoord();
@sqcoord=();
sort_qcoord();

if($verb) { print STDERR "Sort complete\n";}

if($verb) {
    print STDERR "Sorted Rcoords:\n";
    for(my $i=1;$i<=$rcontigno;$i++) {
	for(my $j=0;$j<=$rcoordno[$i];$j++) {
	    print STDERR "$i $j ",$srcoord[$i][$j]," ",$rcoord[$i][$srcoord[$i][$j]][0]," ",$rcoord[$i][$srcoord[$i][$j]][1]," ",$rcoord[$i][$srcoord[$i][$j]][2]," ",$rcoord[$i][$srcoord[$i][$j]][3]," ",$rcoord[$i][$srcoord[$i][$j]][4]," ",$rcoord[$i][$srcoord[$i][$j]][5]," ",$rcoord[$i][$srcoord[$i][$j]][6],"\n";
	}
    }
    print STDERR "Sorted Qcoords:\n";
    for(my $i=1;$i<=$qcontigno;$i++) {
	for(my $j=0;$j<=$qcoordno[$i];$j++) {
	    print STDERR "$i $j ",$sqcoord[$i][$j]," ",$qcoord[$i][$sqcoord[$i][$j]][0]," ",$qcoord[$i][$sqcoord[$i][$j]][1]," ",$qcoord[$i][$sqcoord[$i][$j]][2]," ",$qcoord[$i][$sqcoord[$i][$j]][3]," ",$qcoord[$i][$sqcoord[$i][$j]][4]," ",$qcoord[$i][$sqcoord[$i][$j]][5]," ",$qcoord[$i][$sqcoord[$i][$j]][6],"\n";
	}
    }
}

# - redo the parse of coordinates
$bigcolor=parse_regions($verb);

if($verb) { print STDERR "Parse complete. Now printing.\n";}

# - print resulted regions with different conflict
$conflict="   overlap   ";
print_regions($conflict);
 
if($outfilename) { close(O); select STDOUT;}
   
#*********************************
# SUBROUTINES
#*********************************

#*********************************
# check if two rcoords intersect
#*********************************
sub intersect_r {
    my ($rno,$coord1,$coord2)=@_;

    if($rcoord[$rno][$coord1][0]<=$rcoord[$rno][$coord2][0] && $rcoord[$rno][$coord2][0]<=$rcoord[$rno][$coord1+1][0]) {
	return(1);
    }

    if($rcoord[$rno][$coord2][0]<=$rcoord[$rno][$coord1][0] && $rcoord[$rno][$coord1][0]<=$rcoord[$rno][$coord2+1][0]) {
	return(1);
    }

    return(0);
}

#*********************************
# check if two qcoords intersect
#*********************************
sub intersect_q {
    my ($qno,$coord1,$coord2)=@_;

    if($qcoord[$qno][$coord1][0]<=$qcoord[$qno][$coord2][0] && $qcoord[$qno][$coord2][0]<=$qcoord[$qno][$coord1+1][0]) {
	return(1);
    }

    if($qcoord[$qno][$coord2][0]<=$qcoord[$qno][$coord1][0] && $qcoord[$qno][$coord1][0]<=$qcoord[$qno][$coord2+1][0]) {
	return(1);
    }

    return(0);
}

#*********************************
# mark overlapping conflicts with color
# procedure: sort according to length, and mark shorter overlapping ones 
#*********************************
sub paint_overlap_conflicts() {
    my ($color)=@_;
    my %len;
    my %sim;

    my $contig;

    # first find overlaps in rcoord
    foreach $contig (keys %rcontig) {
	my $rno=$rcontig{$contig};

	my @lensort;
	my $lenno=0;
	%len=();
	%sim=();

	my $i=0;
	while($i<=$rcoordno[$rno]) {

	    if($rcoord[$rno][$srcoord[$rno][$i]][1]==0 && $rcoord[$rno][$srcoord[$rno][$i]][3]==0) { # start of the alignment
		$lensort[$lenno++]=$i;
		$len{$i}=$rcoord[$rno][$srcoord[$rno][$i]+1][0]-$rcoord[$rno][$srcoord[$rno][$i]][0]+1;
		$sim{$i}=$rcoord[$rno][$srcoord[$rno][$i]][7];

		my $qctgno=$rcoord[$rno][$srcoord[$rno][$i]][4];
		my $qcoordno=$rcoord[$rno][$srcoord[$rno][$i]][5];

		if($qcoord[$qctgno][$qcoordno][1]) { # links to end of alignment
		    $len{$i}+=$qcoord[$qctgno][$qcoordno][0]-$qcoord[$qctgno][$qcoordno-1][0]+1;
		}
		else { # links to begining of alignment
		    $len{$i}+=$qcoord[$qctgno][$qcoordno+1][0]-$qcoord[$qctgno][$qcoordno][0]+1;
		}
	    }   

	    $i++;
	}

	# sort according to length
	my @slensort = sort { $len{$a} <=> $len{$b} } @lensort;


	# eliminate overlapping aignments until no more to eliminate
	for(my $j=$lenno-1;$j>=0;$j--) {

	    my $coord=$slensort[$j];

#	    print STDERR "Sort: $contig $rno ",$rcoord[$rno][$srcoord[$rno][$coord]][0]," ",$rcoord[$rno][$srcoord[$rno][$coord]][6]," ",$len{$coord}," ",$sim{$coord},"\n";

	    # check if this alignment wasn't already eliminated;
	    if($rcoord[$rno][$srcoord[$rno][$coord]][6]!=$color) { # this is the biggest alignment that didn't get 
		                                                   # eliminated from previous ones;
		# I need to eliminate the hits which overlap this one but might be longer
		my $i=$j-1;
		my @toelim=();
		my $eliminated=0;
		my $i1=$srcoord[$rno][$coord];
		while($i>=0 && !$eliminated && ($len{$slensort[$i]} >= $len{$coord}-$allowedovlp)) {
                    my $i2=$srcoord[$rno][$slensort[$i]];
		    if($rcoord[$rno][$i2][6]!=$color && intersect_r($rno,$i1,$i2)) {
			if($sim{$coord}>=$sim{$slensort[$i]}) { 
			    push(@toelim,$i2);
			}
			else { 
			    $rcoord[$rno][$i1][6]=$color; # begin aignm
			    my $qctgno=$rcoord[$rno][$i1][4];
			    my $qcoordno=$rcoord[$rno][$i1][5];
			    $qcoord[$qctgno][$qcoordno][6]=$color;
			    $rcoord[$rno][$i1+1][6]=$color; # end alignm
			    $qcoordno=$rcoord[$rno][$i1+1][5];
			    $qcoord[$qctgno][$qcoordno][6]=$color;
			    $eliminated=1;
			}
		    }
		    $i--;
		}

		if(!$eliminated) {

		    for($i=0;$i<=$#toelim;$i++) {
			$rcoord[$rno][$toelim[$i]][6]=$color; # begin aignm
			my $qctgno=$rcoord[$rno][$toelim[$i]][4];
			my $qcoordno=$rcoord[$rno][$toelim[$i]][5];
			$qcoord[$qctgno][$qcoordno][6]=$color;
			$rcoord[$rno][$toelim[$i]+1][6]=$color; # end alignm
			$qcoordno=$rcoord[$rno][$toelim[$i]+1][5];
			$qcoord[$qctgno][$qcoordno][6]=$color;
		    }

		    $i=$coord+1;
		    while($srcoord[$rno][$i]!=$srcoord[$rno][$coord]+1) {
			# eliminate this alignment if in conflict
		    
			if($rcoord[$rno][$srcoord[$rno][$i]][6]!=$color && $rcoord[$rno][$srcoord[$rno][$i]][3]==0) { # && conflictr($rcoord[$rno][$srcoord[$rno][$coord]][2],$rcoord[$rno][$srcoord[$rno][$coord]][4],$rcoord[$rno][$srcoord[$rno][$coord]][5],$rcoord[$rno][$srcoord[$rno][$i]][2],$rcoord[$rno][$srcoord[$rno][$i]][4],$rcoord[$rno][$srcoord[$rno][$i]][5])) {
			
			    $rcoord[$rno][$srcoord[$rno][$i]][6]=$color;


			    my $qctgno=$rcoord[$rno][$srcoord[$rno][$i]][4];
			    my $qcoordno=$rcoord[$rno][$srcoord[$rno][$i]][5];
			    $qcoord[$qctgno][$qcoordno][6]=$color;
			    
			    if($rcoord[$rno][$srcoord[$rno][$i]][1]) { # end of alignment
				$rcoord[$rno][$srcoord[$rno][$i]-1][6]=$color;
				$qctgno=$rcoord[$rno][$srcoord[$rno][$i]-1][4];
				$qcoordno=$rcoord[$rno][$srcoord[$rno][$i]-1][5];
				$qcoord[$qctgno][$qcoordno][6]=$color;
			    }
			    else {
				$rcoord[$rno][$srcoord[$rno][$i]+1][6]=$color;
				$qctgno=$rcoord[$rno][$srcoord[$rno][$i]+1][4];
				$qcoordno=$rcoord[$rno][$srcoord[$rno][$i]+1][5];
				$qcoord[$qctgno][$qcoordno][6]=$color;
			    }
			}
		       
			$i++;
		    }
		}
	    }
	}
    }


    # second find overlaps in qcoord
    foreach $contig (keys %qcontig) {
	my $qno=$qcontig{$contig};

	my @lensort;
	my $lenno=0;
	%len=();
	%sim=();

	my $i=0;
	while($i<=$qcoordno[$qno]) {
	    
	    if($qcoord[$qno][$sqcoord[$qno][$i]][1]==0 && $qcoord[$qno][$sqcoord[$qno][$i]][3]==0) { # start of the alignment
		$lensort[$lenno++]=$i;
		$len{$i}=$qcoord[$qno][$sqcoord[$qno][$i]+1][0]-$qcoord[$qno][$sqcoord[$qno][$i]][0]+1;

		my $rctgno=$qcoord[$qno][$sqcoord[$qno][$i]][4];
		my $rcoordno=$qcoord[$qno][$sqcoord[$qno][$i]][5];

		$sim{$i}=$rcoord[$rctgno][$rcoordno][7];

		if($rcoord[$rctgno][$rcoordno][1]) { # links to end of alignment
		    $len{$i}+=$rcoord[$rctgno][$rcoordno][0]-$rcoord[$rctgno][$rcoordno-1][0]+1;
		}
		else { # links to begining of alignment
		    $len{$i}+=$rcoord[$rctgno][$rcoordno+1][0]-$rcoord[$rctgno][$rcoordno][0]+1;
		}
	    }

	    $i++;
	}

	# sort according to length
	my @slensort = sort { $len{$a} <=> $len{$b} } @lensort;

	# eliminate overlapping alignments until no more to eliminate
	for(my $j=$lenno-1;$j>=0;$j--) {

	    my $coord=$slensort[$j];
	    # check if this alignment wasn't already eliminated;
	    if($qcoord[$qno][$sqcoord[$qno][$coord]][6]!=$color) { # this is the biggest alignment that didn't get 
		                                                   # eliminated from previous ones;
		# I need to eliminate the conflicting ones which overlap this one
		my $i=$j-1;
		my @toelim=();
		my $eliminated=0;
		my $i1=$sqcoord[$qno][$coord];
		while($i>=0 && !$eliminated && $len{$slensort[$i]} >= $len{$coord}-$allowedovlp) {
                    my $i2=$sqcoord[$qno][$slensort[$i]];
		    if($qcoord[$qno][$i2][6]!=$color && intersect_q($qno,$i1,$i2)) {
			if($sim{$coord}>=$sim{$slensort[$i]}) { 
			    push(@toelim,$i2);
			}
			else {
			    $qcoord[$qno][$i1][6]=$color; # begin aignm
			    my $rctgno=$qcoord[$qno][$i1][4];
			    my $rcoordno=$qcoord[$qno][$i1][5];
			    $rcoord[$rctgno][$rcoordno][6]=$color;
			    $qcoord[$qno][$i1+1][6]=$color; # end alignm
			    $rcoordno=$qcoord[$qno][$i1+1][5];
			    $rcoord[$rctgno][$rcoordno][6]=$color;
			    $eliminated=1;
			}
		    }
		    $i--;
		}

		if(!$eliminated) {
		    for($i=0;$i<=$#toelim;$i++) {
			$qcoord[$qno][$toelim[$i]][6]=$color; # begin alignm
			my $rctgno=$qcoord[$qno][$toelim[$i]][4];
			my $rcoordno=$qcoord[$qno][$toelim[$i]][5];
			$rcoord[$rctgno][$rcoordno][6]=$color;
			$qcoord[$qno][$toelim[$i]+1][6]=$color; # end alignm
			$rcoordno=$qcoord[$qno][$toelim[$i]+1][5];
			$rcoord[$rctgno][$rcoordno][6]=$color;
		    }

		    $i=$coord+1;
		    while($sqcoord[$qno][$i]!=$sqcoord[$qno][$coord]+1) {
			# eliminate this alignment if in conflict
		    
			if($qcoord[$qno][$sqcoord[$qno][$i]][6]!=$color && $qcoord[$qno][$sqcoord[$qno][$i]][3]==0) { # && conflictq($qcoord[$qno][$sqcoord[$qno][$coord]][2],$qcoord[$qno][$sqcoord[$qno][$coord]][4],$qcoord[$qno][$sqcoord[$qno][$coord]][5],$qcoord[$qno][$sqcoord[$qno][$i]][2],$qcoord[$qno][$sqcoord[$qno][$i]][4],$qcoord[$qno][$sqcoord[$qno][$i]][5])) {

			    $qcoord[$qno][$sqcoord[$qno][$i]][6]=$color;
			    my $rctgno=$qcoord[$qno][$sqcoord[$qno][$i]][4];
			    my $rcoordno=$qcoord[$qno][$sqcoord[$qno][$i]][5];
			    $rcoord[$rctgno][$rcoordno][6]=$color;
			
			    if($qcoord[$qno][$sqcoord[$qno][$i]][1]) { # end of alignment
				$qcoord[$qno][$sqcoord[$qno][$i]-1][6]=$color;
				$rctgno=$qcoord[$qno][$sqcoord[$qno][$i]-1][4];
				$rcoordno=$qcoord[$qno][$sqcoord[$qno][$i]-1][5];
				$rcoord[$rctgno][$rcoordno][6]=$color;
			    }
			    else {
				$qcoord[$qno][$sqcoord[$qno][$i]+1][6]=$color;
				$rctgno=$qcoord[$qno][$sqcoord[$qno][$i]+1][4];
				$rcoordno=$qcoord[$qno][$sqcoord[$qno][$i]+1][5];
				$rcoord[$rctgno][$rcoordno][6]=$color;
			    }
			}
		       
			$i++;
		    }
		}
	    }
	}
    }
}

#*********************************
# delete all alignments/genes marked with a color bigger then 0 (i.e. delete nonconflicting ones)
#*********************************
sub delete_ok_contigs() {
    my $contig;

    foreach $contig (keys %rcontig) {

	my $rno=$rcontig{$contig};

	my $i=0;

	# delete both start and end of the alignment/gene
	while($rcoordno[$rno]>0 && $rcoord[$rno][$rcoordno[$rno]][6]>0) {
	    $rcoord[$rno][$rcoordno[$rno]-1]=();
	    $rcoord[$rno][$rcoordno[$rno]]=();
	    undef $rcoord[$rno][$rcoordno[$rno]-1];
	    undef $rcoord[$rno][$rcoordno[$rno]];
	    $rcoordno[$rno]-=2;
	}

	while($i<=$rcoordno[$rno]) {

	    my $color=$rcoord[$rno][$i][6];

	    if($color==-1) {
		$rcoord[$rno][$i][6]=0;
	    }
	    elsif($color>0) {
                # this variant requires a manual re-sort; it might be better then using splice because 
                # otherwise I need to redo all links between rcoord and qcoord
		if($i<$rcoordno[$rno]-1) {

		    copy_rcoord($rno,$i,$rcoordno[$rno]-1);
		    copy_rcoord($rno,$i+1,$rcoordno[$rno]);
		    
		    # update qcoord if alignment
		    if($rcoord[$rno][$i][3]==0) {
			my $qctgno=$rcoord[$rno][$i][4];
			my $qcoordno=$rcoord[$rno][$i][5];

			$qcoord[$qctgno][$qcoordno][5]=$i;
			$qcoordno=$rcoord[$rno][$i+1][5];
			$qcoord[$qctgno][$qcoordno][5]=$i+1;
		    }

		    $i--;

		}
		do {
		    $rcoord[$rno][$rcoordno[$rno]-1]=();
		    $rcoord[$rno][$rcoordno[$rno]]=();
		    undef $rcoord[$rno][$rcoordno[$rno]-1];
		    undef $rcoord[$rno][$rcoordno[$rno]];
		    $rcoordno[$rno]-=2;
		} while($rcoordno[$rno]>0 && $rcoord[$rno][$rcoordno[$rno]][6]>0);
	    }
	    $i++;
	}
    }

    foreach $contig (keys %qcontig) {

	my $qno=$qcontig{$contig};

	while($qcoordno[$qno]>0 && $qcoord[$qno][$qcoordno[$qno]][6]>0) {
	    $qcoord[$qno][$qcoordno[$qno]-1]=();
	    $qcoord[$qno][$qcoordno[$qno]]=();
	    undef $qcoord[$qno][$qcoordno[$qno]-1];
	    undef $qcoord[$qno][$qcoordno[$qno]];
	    $qcoordno[$qno]-=2;
	}

	my $i=0;
	while($i<=$qcoordno[$qno]) {

	    my $color=$qcoord[$qno][$i][6];

	    if($color==-1) {
		$qcoord[$qno][$i][6]=0;
	    }
	    elsif($color>0) {
		if($i<$qcoordno[$qno]-1) {
		    # delete both start and end of the alignment
		    copy_qcoord($qno,$i,$qcoordno[$qno]-1);
		    copy_qcoord($qno,$i+1,$qcoordno[$qno]);
		    
		    # update rcoord if alignment
		    if($qcoord[$qno][$i][3]==0) {
			my $rctgno=$qcoord[$qno][$i][4];
			my $rcoordno=$qcoord[$qno][$i][5];

			$rcoord[$rctgno][$rcoordno][5]=$i;
			$rcoordno=$qcoord[$qno][$i+1][5];
			$rcoord[$rctgno][$rcoordno][5]=$i+1;
		    }
		    $i--;

		}
		do {
		    $qcoord[$qno][$qcoordno[$qno]-1]=();
		    $qcoord[$qno][$qcoordno[$qno]]=();
		    undef $qcoord[$qno][$qcoordno[$qno]-1];
		    undef $qcoord[$qno][$qcoordno[$qno]];
		    $qcoordno[$qno]-=2;
		} while($qcoordno[$qno]>0 && $qcoord[$qno][$qcoordno[$qno]][6]>0);
	    }
	    $i++;
	}
    }

}

#*********************************
# sort rcoord according to value
#*********************************
sub sort_rcoord {
    my @nrcoord;
    for(my $i=1;$i<=$rcontigno;$i++) { # sorting is done in each contig $i
	for(my $j=0;$j<=$rcoordno[$i];$j++) {
	    $nrcoord[$i][$j]=$j;
	}
	# sort according to coordinate value (first position)
	if($rcoordno[$i]>=0) {
	    @{$srcoord[$i]} = sort { $rcoord[$i][$a][0] <=> $rcoord[$i][$b][0]} @{$nrcoord[$i]};
	    @{$nrcoord[$i]}=();
	}
    }
}

#*********************************
# sort qcoord according to value
#*********************************
sub sort_qcoord {
    my @nqcoord;
    for(my $i=1;$i<=$qcontigno;$i++) {
	for(my $j=0;$j<=$qcoordno[$i];$j++) {
	    $nqcoord[$i][$j]=$j;
	}
	if($qcoordno[$i]>=0) {
	    @{$sqcoord[$i]} = sort { $qcoord[$i][$a][0] <=> $qcoord[$i][$b][0]} @{$nqcoord[$i]};
	    @{$nqcoord[$i]}=();
	}
    }
}

#*********************************
# read all data into the coordinates arrays
#*********************************
sub read_coords {

    my ($refquecoords,$cds1,$cds2,$verb)=@_;
    my %refer;        # each alignment is kept here for later reference


    # look into coords file and fill in rcoords
    open(F,$refquecoords);
    <F>; # 5 lines of no interest; so far...
    <F>; # ...
    <F>;
    <F>;
    <F>;
    while(<F>) {
	chomp;
	my @a=split;
    
	my ($minval,$maxval,$sign,$alignment);

	if(!$rcontig{$a[-2]}) { # new contig to store
	    $rcontig{$a[-2]}=++$rcontigno;
	    $rlcontig[$rcontigno]=$a[$ind_lenref];
	    $rcoordno[$rcontigno]=0;
	    if($verb) { print STDERR $a[-2]," ",$rcontig{$a[-2]},"\n";}

	}
	else { $rcoordno[$rcontig{$a[-2]}]++;}

	# fill rcoord fields
	if($a[$ind_s1]<$a[$ind_e1]) {
	    $minval=$a[$ind_s1];
	    $maxval=$a[$ind_e1];
	    $sign=1;
	}
	else {
	    $minval=$a[$ind_e1];
	    $maxval=$a[$ind_s1];
	    $sign=-1;
	}

	# store alignment for later reference
	$alignment="[".$a[-2]."][".$a[-1]."]".$a[$ind_s1].".".$a[$ind_e1].".".$a[$ind_s2].".".$a[$ind_e2];
	$refer{$alignment}=$rcoordno[$rcontig{$a[-2]}];

	# start=0|end=1; +=1|-=-1; alignment=0|gene=1; nocolor=0;
	# rcoord = (value, start|end, +|-, alignment|gene, qcoordlink-contigno, qcoordlink-coordno, color, PID)

	# PID  : NOT YET USED
	# color is 0 while no asignments to a syntenic region has been made; 
	# color is -1 if a conflict inside region has been found
	
	@{$rcoord[$rcontig{$a[-2]}][$rcoordno[$rcontig{$a[-2]}]++]}=($minval,0,$sign,0,-1,-1,0,$a[$ind_psim]);
	@{$rcoord[$rcontig{$a[-2]}][$rcoordno[$rcontig{$a[-2]}]]}=($maxval,1,$sign,0,-1,-1,0,$a[$ind_psim]);
#	@{$rcoord[$rcontig{$a[-2]}][$rcoordno[$rcontig{$a[-2]}]++]}=($minval,0,$sign,0,-1,-1,0);
#	@{$rcoord[$rcontig{$a[-2]}][$rcoordno[$rcontig{$a[-2]}]]}=($maxval,1,$sign,0,-1,-1,0);
    }
    close(F);


    # look into coords file and fill in qcoords
    open(F,$refquecoords);
    <F>;
    <F>;
    <F>;
    <F>;
    <F>;

    while(<F>) {
	chomp;
	my @a=split;
	
	my ($minval,$maxval,$sign,$alignment);
    
	if(!$qcontig{$a[-1]}) { # new contig to store
	    $qcontig{$a[-1]}=++$qcontigno;
	    $qlcontig[$qcontigno]=$a[$ind_lenque];
	    $qcoordno[$qcontigno]=0;
	    if($verb) { print STDERR $a[-1]," ",$qcontig{$a[-1]},"\n";}
	}
	else { $qcoordno[$qcontig{$a[-1]}]++;}

	# fill qcoord fields
	if($a[$ind_s2]<$a[$ind_e2]) {
	    $minval=$a[$ind_s2];
	    $maxval=$a[$ind_e2];
	    $sign=1;
	}
	else {
	    $minval=$a[$ind_e2];
	    $maxval=$a[$ind_s2];
	    $sign=-1;
	}

	# ckeck alignment for link reference
	$alignment="[".$a[-2]."][".$a[-1]."]".$a[$ind_s1].".".$a[$ind_e1].".".$a[$ind_s2].".".$a[$ind_e2];
	my $rctgno=$rcontig{$a[-2]};    # retrieve reference contig no and coord no for given alignment
	my $rcrdno=$refer{$alignment};
    
	# start=0|end=1; +=1|-=-1; alignment=0|gene=1; nocolor=0;
	# qcoord = (0:value, 1:start|end, 2:+|-, 3:alignment|gene, 4:rcoordlink-contigno, 5:rcoordlink-coordno, 6:color 7:psim)
	@{$qcoord[$qcontig{$a[-1]}][$qcoordno[$qcontig{$a[-1]}]++]}=($minval,0,$sign,0,$rctgno,$rcrdno,0);
	@{$qcoord[$qcontig{$a[-1]}][$qcoordno[$qcontig{$a[-1]}]]}=($maxval,1,$sign,0,$rctgno,$rcrdno+1,0);
    
	# update rcoord with link info
	$rcoord[$rctgno][$rcrdno][4]=$qcontig{$a[-1]};
	$rcoord[$rctgno][$rcrdno][5]=$qcoordno[$qcontig{$a[-1]}]-1;
	$rcoord[$rctgno][$rcrdno+1][4]=$qcontig{$a[-1]};
	$rcoord[$rctgno][$rcrdno+1][5]=$qcoordno[$qcontig{$a[-1]}];
    }
    close(F);   

    %refer=(); # clean memory

    # read in genes if present

    my $maxvalall=0; # keeps maximum seen coordinate of a gene in a contig -> to ensure no overlaping genes are given

    my %newcontig;   # remembers seen contigs with genes

    # genes from reference
    if($cds1) {

	my %transcript; # remembers seen transcripts
	my $first=1;    # first line in file -> to ensure that a transcript has been seen
	my ($minval,$maxval,$exons,$contig,$sign);
	
	open(F,$cds1);
	while(<F>) {
	    chomp;
	    my @a=split;
	    my $i=$#a;
	    while(!($a[$i] =~ /transcript_id/)) { $i--;}
	    if(!$transcript{$a[$i+1]}) { # new transcript
		# minval>maxvalall to make sure there is only one gene per region
		# this program supposes there are no alternative splice forms
		if(!$first && $minval>$maxvalall) {
		    if($rcontig{$contig}) { # store genes only from contigs that have promer hits
			@{$rcoord[$rcontig{$contig}][++$rcoordno[$rcontig{$contig}]]}=($minval,0,$sign,1,$exons,-1,0);
			@{$rcoord[$rcontig{$contig}][++$rcoordno[$rcontig{$contig}]]}=($maxval,1,$sign,1,-1,-1,0);
		    }
		}
		else { 
		    $first=0;
		}
		if(!$newcontig{$a[0]}) { $maxvalall=0; $newcontig{$a[0]}++;}
		elsif($maxval>$maxvalall) { $maxvalall = $maxval; } 
		$contig=$a[0];
		$sign = $a[6] eq "+" ? 1 : -1;
		$transcript{$a[$i+1]}++;
		$minval=$a[3];
		$exons="";
	    }
	    $exons.=$a[3]." ".$a[4]." ";
	    $maxval=$a[4];
	}
	# store last seen transcript 
	if($minval > $maxvalall && $rcontig{$contig}) {
	    @{$rcoord[$rcontig{$contig}][++$rcoordno[$rcontig{$contig}]]}=($minval,0,$sign,1,$exons,-1,0);
	    @{$rcoord[$rcontig{$contig}][++$rcoordno[$rcontig{$contig}]]}=($maxval,1,$sign,1,-1,-1,0);
	}
	close(F);
    }

    %newcontig=();

    $maxvalall=0;

    # genes from query; 
    if($cds2) {
	my %transcript;
	my $first=1; 
	my ($minval,$maxval,$exons,$contig,$sign);

	open(F,$cds2);
	while(<F>) {
	    chomp;
	    my @a=split;
	    my $i=$#a;
	    while(!($a[$i] =~ /transcript_id/)) { $i--;}
	    if(!$transcript{$a[$i+1]}) {
		if(!$first && $minval>$maxvalall) {
		    if($qcontig{$contig}) {
			@{$qcoord[$qcontig{$contig}][++$qcoordno[$qcontig{$contig}]]}=($minval,0,$sign,1,$exons,-1,0);
			@{$qcoord[$qcontig{$contig}][++$qcoordno[$qcontig{$contig}]]}=($maxval,1,$sign,1,-1,-1,0);
		    }
		}
		else {
		    $first=0;
		}
		if(!$newcontig{$a[0]}) { $maxvalall=0; $newcontig{$a[0]}++;}
		elsif($maxval>$maxvalall) { $maxvalall = $maxval; }
		$contig=$a[0];
		$sign = $a[6] eq "+" ? 1 : -1;
		$transcript{$a[$i+1]}++;
		$minval=$a[3];
		$exons="";
	    }
	    $exons.=$a[3]." ".$a[4]." ";
	    $maxval=$a[4];
	}
	if($qcontig{$contig}) {
	    @{$qcoord[$qcontig{$contig}][++$qcoordno[$qcontig{$contig}]]}=($minval,0,$sign,1,$exons,-1,0);
	    @{$qcoord[$qcontig{$contig}][++$qcoordno[$qcontig{$contig}]]}=($maxval,1,$sign,1,-1,-1,0);
	}
	close(F);
    }

    %newcontig=(); # clear memory
}


#*********************************
# copy second rcoord item into the second
#*********************************
sub copy_rcoord {
    my ($ctgno,$coord1,$coord2)=@_;

    for(my $i=0;$i<8;$i++) { # when psim is used
#    for(my $i=0;$i<7;$i++) {
	$rcoord[$ctgno][$coord1][$i]=$rcoord[$ctgno][$coord2][$i];
    }
}

#*********************************
# copy second qcoord item into the second
#*********************************
sub copy_qcoord {
    my ($ctgno,$coord1,$coord2)=@_;

    for(my $i=0;$i<7;$i++) {
	$qcoord[$ctgno][$coord1][$i]=$qcoord[$ctgno][$coord2][$i];
    }
}

#*********************************
# determine non-conflicting regions; conflicting regions are marked with the color -1
#*********************************
sub parse_regions {
    
    my ($verb)=@_;

    if($verb) { print STDERR "Parsing rcoords:\n"; }

    # parse rcoord
    my $color=1;
    for(my $i=1;$i<=$rcontigno;$i++) {
	if($verb) { print STDERR "Contig no: $i\n";}
	my $region_sign=0; # no region seen yet; otherwise is +/-
	my $region_link=0; # no region seen yet; otherwise is >= 1
	my $region_link_coord=0;
	my $start_gene=0;       # no gene started
	my $start_alignment=0;  # no alignment started
	my $n=0; # n goes through all items stored in $rcoord[$i] in the srcoord order
	my $newcolor;
	while($n<=$rcoordno[$i]) {

	    if($verb) { 
		print STDERR "Alignm: ",$srcoord[$i][$n]," ",$rcoord[$i][$srcoord[$i][$n]][0]," ",$rcoord[$i][$srcoord[$i][$n]][1]," ",$rcoord[$i][$srcoord[$i][$n]][2]," ",$rcoord[$i][$srcoord[$i][$n]][3]," ",$rcoord[$i][$srcoord[$i][$n]][4]," ",$rcoord[$i][$srcoord[$i][$n]][5]," ",$rcoord[$i][$srcoord[$i][$n]][6],"\n";
		print STDERR "Region: $region_sign,$region_link,$region_link_coord\n";
	    }

	    # check all possible points
	    if($rcoord[$i][$srcoord[$i][$n]][3]==0) { # deal with alignment
		if($rcoord[$i][$srcoord[$i][$n]][1]==0) { # start of alignment
		    if($verb) { print STDERR "start_gene=$start_gene\n";}
		    if($start_gene) { # gene already started
			my $j=$n-1; # j will store start of gene
			while($rcoord[$i][$srcoord[$i][$j]][3]==0 || $rcoord[$i][$srcoord[$i][$j]][1]==1 ) { $j--;}
			if($rcoord[$i][$srcoord[$i][$j]][2] != $rcoord[$i][$srcoord[$i][$n]][2]) { # conflict w/ gene 
			    $newcolor=-1;
			    $region_sign=$rcoord[$i][$srcoord[$i][$n]][2];
			    $region_link=$rcoord[$i][$srcoord[$i][$n]][4];
			    $region_link_coord=$rcoord[$i][$srcoord[$i][$n]][5];
			    $color++;
			}
			elsif(conflictr($region_sign,$region_link,$region_link_coord,$rcoord[$i][$srcoord[$i][$n]][2],$rcoord[$i][$srcoord[$i][$n]][4],$rcoord[$i][$srcoord[$i][$n]][5])) {
			    if($verb) { print STDERR "Conflictr: $region_sign,$region_link,$region_link_coord,",$rcoord[$i][$srcoord[$i][$n]][2],",",$rcoord[$i][$srcoord[$i][$n]][4],",",$rcoord[$i][$srcoord[$i][$n]][5],"\n";}
			    $region_sign=$rcoord[$i][$srcoord[$i][$n]][2];
			    $region_link=$rcoord[$i][$srcoord[$i][$n]][4];
			    $region_link_coord=$rcoord[$i][$srcoord[$i][$n]][5];
			    $newcolor=++$color;
			}
			else { # no gene / region conflict
			    $newcolor=$color;
			    if(!$region_link) {
				$region_sign=$rcoord[$i][$srcoord[$i][$n]][2];
				$region_link=$rcoord[$i][$srcoord[$i][$n]][4];
			    }
			    $region_link_coord=$rcoord[$i][$srcoord[$i][$n]][5];
			}

			# now mark gene/alignment with new colors
			my $end_alignment=$srcoord[$i][$n]+1;
			my $ka=$n+1;
			while($srcoord[$i][$ka]!=$end_alignment) { $ka++;}
			my $end_gene=$srcoord[$i][$j]+1;
			my $kg=$j+1;
			while($srcoord[$i][$kg]!=$end_gene) { $kg++;}
			
			my $minmark = $n < $j ? $n : $j;
			my $maxmark = $ka > $kg ? $ka : $kg;
			# mark alignment and gene
			markr($i,$minmark,$maxmark,$newcolor);
		    }
		    else { # no genes started
			if(conflictr($region_sign,$region_link,$region_link_coord,$rcoord[$i][$srcoord[$i][$n]][2],$rcoord[$i][$srcoord[$i][$n]][4],$rcoord[$i][$srcoord[$i][$n]][5])) {
			    if($verb) { print STDERR "Conflictr: $region_sign,$region_link,$region_link_coord,",$rcoord[$i][$srcoord[$i][$n]][2],",",$rcoord[$i][$srcoord[$i][$n]][4],",",$rcoord[$i][$srcoord[$i][$n]][5],"\n";}
			    $region_sign=$rcoord[$i][$srcoord[$i][$n]][2];
			    $region_link=$rcoord[$i][$srcoord[$i][$n]][4];
			    $region_link_coord=$rcoord[$i][$srcoord[$i][$n]][5];
			    $newcolor=++$color;
			}
			else { 
			    if(!$region_link) {
				$region_sign=$rcoord[$i][$srcoord[$i][$n]][2];
				$region_link=$rcoord[$i][$srcoord[$i][$n]][4];
			    }
			    $newcolor=$color;
			    $region_link_coord=$rcoord[$i][$srcoord[$i][$n]][5];
			}
			my $end_alignment=$srcoord[$i][$n]+1;
			my $ka=$n+1;
			while($srcoord[$i][$ka]!=$end_alignment) { $ka++;}
			markr($i,$n,$ka,$newcolor);
		    }
		    $start_alignment++;
		}
		elsif($rcoord[$i][$srcoord[$i][$n]][1]==1) { # end of alignment
		    $start_alignment--;
		    if($verb) { print STDERR "start_alignm=$start_alignment\n";}
		    my $end_alignment=$srcoord[$i][$n]-1;
		    my $ka=$n-1;
		    while($srcoord[$i][$ka]!=$end_alignment) { $ka--;}
		    if(conflictr($region_sign,$region_link,$region_link_coord,$rcoord[$i][$srcoord[$i][$n]][2],$rcoord[$i][$srcoord[$i][$n]][4],$rcoord[$i][$srcoord[$i][$n]][5])) {
			if($verb) { print STDERR "Conflictr: $region_sign,$region_link,$region_link_coord,",$rcoord[$i][$srcoord[$i][$n]][2],",",$rcoord[$i][$srcoord[$i][$n]][4],",",$rcoord[$i][$srcoord[$i][$n]][5],"\n";}
			markr($i,$ka,$n,-1);
			if(!$start_alignment) { # stop region
			    $color++;
			    $region_sign=0; # no region started
			    $region_link=0; # no region started
			    $region_link_coord=0;
			}
		    }
		    else {
			if(checkr($i,$ka,$n,-1)) { # alignment has -1's
			    markr($i,$ka,$n,-1);
			    if(!$start_alignment) { # reset region
				$color++;
				$region_sign=0; # no region started
				$region_link=0; # no region started
				$region_link_coord=0;
			    }	
			}
			elsif($start_gene) {
			    if($verb) { print STDERR "start_gene=$start_gene\n";}
			    my $j=$n-1; # j will store start of gene
			    while($rcoord[$i][$srcoord[$i][$j]][3]==0 || $rcoord[$i][$srcoord[$i][$j]][1]==1 ) { $j--;}
			    my $kg=$n+1; # kg will store end of gene
			    while($rcoord[$i][$srcoord[$i][$kg]][3]==0 || $rcoord[$i][$srcoord[$i][$kg]][1]==0 ) { $kg++;}
			    markr($i,$j,$kg,$rcoord[$i][$srcoord[$i][$n]][6]);
			}
		    }
		}
	    }
	    elsif($rcoord[$i][$srcoord[$i][$n]][3]==1) { # deal with gene
		if($rcoord[$i][$srcoord[$i][$n]][1]==0) { # start of gene
		    $start_gene=1;
		    if(!$start_alignment) {
			if($region_sign!=0 && $region_sign != $rcoord[$i][$srcoord[$i][$n]][2] ) { # reset region - only if conflict with region
			    $color++;
			    $region_sign=0; # no region started
			    $region_link=0; # no region started
			    $region_link_coord=0;
			}
		    }	
		    else {
			if(!$region_sign || $region_sign == $rcoord[$i][$srcoord[$i][$n]][2] ) { $newcolor=$color; }
			else { $newcolor =-1;}
			my $end_gene=$srcoord[$i][$n]+1;
			my $kg=$n+1;
			while($srcoord[$i][$kg]!=$end_gene) { $kg++;}
			markr($i,$n,$kg,$newcolor);
		    }
		}
		elsif($rcoord[$i][$srcoord[$i][$n]][1]==1) { # end of gene
		    $start_gene--;
		    my $end_gene=$srcoord[$i][$n]-1;
		    my $kg=$n-1;
		    while($srcoord[$i][$kg]!=$end_gene) { $kg--;}
		    if($region_sign!=0 && $region_sign!=$rcoord[$i][$srcoord[$i][$n]][2]) { # conflict with region
			markr($i,$kg,$n,-1);
			if(!$start_alignment) { # stop region
			    $color++;
			    $region_sign=0; # no region started
			    $region_link=0; # no region started
			    $region_link_coord=0;
			}
		    }
		    else {
			if(checkr($i,$kg,$n,-1)) { # alignment has -1's
			    markr($i,$kg,$n,-1);
			    if(!$start_alignment) { # reset region
				$color++;
				$region_sign=0; # no region started
				$region_link=0; # no region started
				$region_link_coord=0;
			    }	
			}
		    }
		}
	    }
	    if($verb) { print STDERR "color=$color\n";}
	    $n++;
	}
    }

    if($verb) {
	print STDERR "Sorted Rcoords one:\n";
	for(my $i=1;$i<=$rcontigno;$i++) {
	    for(my $j=0;$j<=$rcoordno[$i];$j++) {
		print STDERR "$i $j ",$srcoord[$i][$j]," ",$rcoord[$i][$srcoord[$i][$j]][0]," ",$rcoord[$i][$srcoord[$i][$j]][1]," ",$rcoord[$i][$srcoord[$i][$j]][2]," ",$rcoord[$i][$srcoord[$i][$j]][3]," ",$rcoord[$i][$srcoord[$i][$j]][4]," ",$rcoord[$i][$srcoord[$i][$j]][5]," ",$rcoord[$i][$srcoord[$i][$j]][6],"\n";
	    }
	}
    }


    if($verb) { print STDERR "Parsing qcoords:\n";}

    # parse qcoord ; this one is different because the qcoord records already have colors assigned
    for(my $i=1;$i<=$qcontigno;$i++) {

	if($verb) { print STDERR "Contig no: $i with ",$qcoordno[$i],"\n";}

	my $seencolor=$color;
	my $region_sign=0; # no region seen yet; otherwise is +/-
	my $region_link=0; # no region seen yet; otherwise is >= 1
	my $region_link_coord=0;
	my $start_gene=0;       # no gene started
	my $start_alignment=0;  # no alignment started
	my $n=0; # n goes through all items stored in $qcoord[$i] in the sqcoord order
	my $newcolor;
	my %nocolor;
	while($n<=$qcoordno[$i]) {

	    if($verb) { print STDERR "Alignm: $n ",$sqcoord[$i][$n]," ",$qcoord[$i][$sqcoord[$i][$n]][0]," ",$qcoord[$i][$sqcoord[$i][$n]][1]," ",$qcoord[$i][$sqcoord[$i][$n]][2]," ",$qcoord[$i][$sqcoord[$i][$n]][3]," ",$qcoord[$i][$sqcoord[$i][$n]][4]," ",$qcoord[$i][$sqcoord[$i][$n]][5]," ",$qcoord[$i][$sqcoord[$i][$n]][6],"\n";	
			print STDERR "Region: $region_sign,$region_link,$region_link_coord\n";
		    }

	    # check all possible points
	    if($qcoord[$i][$sqcoord[$i][$n]][3]==0) { # deal with alignment
		if($qcoord[$i][$sqcoord[$i][$n]][1]==0) { # start of alignment
		    if($start_gene) { # gene already started
			my $j=$n-1; # j will store start of gene
			while($qcoord[$i][$sqcoord[$i][$j]][3]==0 || $qcoord[$i][$sqcoord[$i][$j]][1]==1 ) { $j--;}
			if($qcoord[$i][$sqcoord[$i][$j]][2] != $qcoord[$i][$sqcoord[$i][$n]][2]) { # conflict w/ gene
			    $newcolor=-1;
			    $region_sign=$qcoord[$i][$sqcoord[$i][$n]][2];
			    $region_link=$qcoord[$i][$sqcoord[$i][$n]][4];
			    $region_link_coord=$qcoord[$i][$sqcoord[$i][$n]][5];
			}
			elsif(conflictq($region_sign,$region_link,$region_link_coord,$qcoord[$i][$sqcoord[$i][$n]][2],$qcoord[$i][$sqcoord[$i][$n]][4],$qcoord[$i][$sqcoord[$i][$n]][5])) {
			    if($verb) { print STDERR "Conflictq: $region_sign,$region_link,$region_link_coord,",$qcoord[$i][$sqcoord[$i][$n]][2],",",$qcoord[$i][$sqcoord[$i][$n]][4],",",$qcoord[$i][$sqcoord[$i][$n]][5],"\n";}
			    if($rcoord[$region_link][$region_link_coord][6] != $qcoord[$i][$sqcoord[$i][$n]][6] && !$nocolor{$qcoord[$i][$sqcoord[$i][$n]][6]}) { 
				$newcolor=$qcoord[$i][$sqcoord[$i][$n]][6];
			    }
			    else { $newcolor=++$seencolor; }
			    $region_sign=$qcoord[$i][$sqcoord[$i][$n]][2];
			    $region_link=$qcoord[$i][$sqcoord[$i][$n]][4];
			    $region_link_coord=$qcoord[$i][$sqcoord[$i][$n]][5];
			}
			else { # no gene / region conflict
			    $newcolor=$qcoord[$i][$sqcoord[$i][$n]][6];
			    if(!$region_link) { # leave color unchanged
				$region_sign=$qcoord[$i][$sqcoord[$i][$n]][2];
				$region_link=$qcoord[$i][$sqcoord[$i][$n]][4];
				if($nocolor{$newcolor}) { $newcolor=++$seencolor;}
			    }
			    else { # keep region's color if nocolor
				if($newcolor!=-1 && $nocolor{$newcolor}) {
				    $newcolor=$rcoord[$region_link][$region_link_coord][6];
				}
			    }
			    $region_link_coord=$qcoord[$i][$sqcoord[$i][$n]][5];
			}
		    
			$nocolor{$qcoord[$i][$sqcoord[$i][$n]][6]}=1;

			# now mark gene/alignment with new colors
			my $end_alignment=$sqcoord[$i][$n]+1;
			my $ka=$n+1;
			while($sqcoord[$i][$ka]!=$end_alignment) { $ka++;}
			my $end_gene=$sqcoord[$i][$j]+1;
			my $kg=$j+1;
			while($sqcoord[$i][$kg]!=$end_gene) { $kg++;}
		
			my $minmark = $n < $j ? $n : $j;
			my $maxmark = $ka > $kg ? $ka : $kg;
			# mark alignment and gene
			if($verb) { print STDERR "1 markq\n";}
			markq($i,$minmark,$maxmark,$newcolor,$color);
		    }
		    else { # no genes started
			if(conflictq($region_sign,$region_link,$region_link_coord,$qcoord[$i][$sqcoord[$i][$n]][2],$qcoord[$i][$sqcoord[$i][$n]][4],$qcoord[$i][$sqcoord[$i][$n]][5])) {
			    if($verb) { print STDERR "conflictq: $region_sign,$region_link,$region_link_coord ",$qcoord[$i][$sqcoord[$i][$n]][2],",",$qcoord[$i][$sqcoord[$i][$n]][4],",",$qcoord[$i][$sqcoord[$i][$n]][5],"\n";}
			    if($rcoord[$region_link][$region_link_coord][6] != $qcoord[$i][$sqcoord[$i][$n]][6] && !$nocolor{$qcoord[$i][$sqcoord[$i][$n]][6]}) { 
				$newcolor=$qcoord[$i][$sqcoord[$i][$n]][6];
			    }
			    else { $newcolor=++$seencolor; }
			    
			    $region_sign=$qcoord[$i][$sqcoord[$i][$n]][2];
			    $region_link=$qcoord[$i][$sqcoord[$i][$n]][4];
			    $region_link_coord=$qcoord[$i][$sqcoord[$i][$n]][5];
			    my $end_alignment=$sqcoord[$i][$n]+1;
			    my $ka=$n+1;
			    while($sqcoord[$i][$ka]!=$end_alignment) { $ka++;}
			    if($verb) { print STDERR "2 markq\n";}
			    markq($i,$n,$ka,$newcolor,$color);
			}
			else { 
			    $newcolor=$qcoord[$i][$sqcoord[$i][$n]][6];
			    if(!$region_link) {
				if($nocolor{$newcolor}) { $newcolor=++$seencolor; }
				$region_sign=$qcoord[$i][$sqcoord[$i][$n]][2];
				$region_link=$qcoord[$i][$sqcoord[$i][$n]][4];
			    } 
			    else { # keep region's color if nocolor
				if($newcolor!=-1 && $nocolor{$newcolor}) {
				    $newcolor=$rcoord[$region_link][$region_link_coord][6];
				}
			    }

			    my $end_alignment=$sqcoord[$i][$n]+1;
			    my $ka=$n+1;
			    while($sqcoord[$i][$ka]!=$end_alignment) { $ka++;}
			    if($verb) { print STDERR "3 markq\n";}
			    markq($i,$n,$ka,$newcolor,$color);
			    $region_link_coord=$qcoord[$i][$sqcoord[$i][$n]][5];
			}
			$nocolor{$qcoord[$i][$sqcoord[$i][$n]][6]}=1;
		    }
		    $start_alignment++;
		}
		elsif($qcoord[$i][$sqcoord[$i][$n]][1]==1) { # end of alignment
		    $start_alignment--;
		    my $end_alignment=$sqcoord[$i][$n]-1;
		    my $ka=$n-1;
		    while($sqcoord[$i][$ka]!=$end_alignment) { $ka--;}
		    if(conflictq($region_sign,$region_link,$region_link_coord,$qcoord[$i][$sqcoord[$i][$n]][2],$qcoord[$i][$sqcoord[$i][$n]][4],$qcoord[$i][$sqcoord[$i][$n]][5])) {
			if($verb) { print STDERR "4 markq\n";}
			markq($i,$ka,$n,-1,$color);
			if(!$start_alignment) { # stop region
			    $seencolor++;
			    $region_sign=0; # no region started
			    $region_link=0; # no region started
			    $region_link_coord=0;
			}
		    }
		    else {
			if(checkq($i,$ka,$n,-1)) { # alignment has -1's
			    if($verb) { print STDERR "5 markq\n";}
			    markq($i,$ka,$n,-1,$color);
			    if(!$start_alignment) { # reset region
				$seencolor++;
				$region_sign=0; # no region started
				$region_link=0; # no region started
				$region_link_coord=0;
			    }	
			}
			elsif($start_gene) {
			    if($verb) { print STDERR "start_gene=$start_gene\n";}
			    my $j=$n-1; # j will store start of gene
			    while($qcoord[$i][$sqcoord[$i][$j]][3]==0 || $qcoord[$i][$sqcoord[$i][$j]][1]==1 ) { $j--;}
			    my $kg=$n+1; # kg will store end of gene
			    while($qcoord[$i][$sqcoord[$i][$kg]][3]==0 || $qcoord[$i][$sqcoord[$i][$kg]][1]==0 ) { $kg++;}
			    if($verb) { print STDERR "6 markq\n";}
			    markq($i,$j,$kg,$qcoord[$i][$sqcoord[$i][$n]][6],$color);
			}
		    }
		}
	    }
	    elsif($qcoord[$i][$sqcoord[$i][$n]][3]==1) { # deal with gene
		if($qcoord[$i][$sqcoord[$i][$n]][1]==0) { # start of gene
		    $start_gene=1;
		    if(!$start_alignment) {
			if($region_sign!=0 && $region_sign != $qcoord[$i][$sqcoord[$i][$n]][2]) { #  reset region - only if conflict with previous region
			    $seencolor++;
			    $region_sign=0; # no region started
			    $region_link=0; # no region started
			    $region_link_coord=0;
			}
		    }	
		    else {
			if(!$region_sign || $region_sign == $qcoord[$i][$sqcoord[$i][$n]][2] ) { 
			    if($region_link) {
				$newcolor=$rcoord[$region_link][$region_link_coord][6];
				if($verb) { print STDERR "newcolor=$newcolor\n";}
				my $end_gene=$sqcoord[$i][$n]+1;
				my $kg=$n+1;
				while($sqcoord[$i][$kg]!=$end_gene) { $kg++;}
				if($verb) { print STDERR "7 markq\n";}
				markq($i,$n,$kg,$newcolor,$color);
			    }
			}
			else { $newcolor =-1; 
			       my $end_gene=$sqcoord[$i][$n]+1;
			       my $kg=$n+1;
			       while($sqcoord[$i][$kg]!=$end_gene) { $kg++;}
			       if($verb) { print STDERR "8 markq\n";}
			       markq($i,$n,$kg,$newcolor,$color);
			   }
		    }
		}
		elsif($qcoord[$i][$sqcoord[$i][$n]][1]==1) { # end of gene
		    $start_gene--;
		    my $end_gene=$sqcoord[$i][$n]-1;
		    my $kg=$n-1;
		    while($sqcoord[$i][$kg]!=$end_gene) { $kg--;}
		    if($region_sign!=0 && $region_sign!=$qcoord[$i][$sqcoord[$i][$n]][2]) { # conflict with region
			if($verb) { print STDERR "9 markq\n";}
			markq($i,$kg,$n,-1,$color);
			if(!$start_alignment) { # stop region
			    $seencolor++;
			    $region_sign=0; # no region started
			    $region_link=0; # no region started
			    $region_link_coord=0;
			}
		    }
		    else {
			if(checkq($i,$kg,$n,-1)) { # alignment has -1's
			    if($verb) { print STDERR "10 markq\n";}
			    markq($i,$kg,$n,-1);
			    if(!$start_alignment) { # reset region
				$seencolor++;
				$region_sign=0; # no region started
				$region_link=0; # no region started
				$region_link_coord=0;
			    }	
			}
		    }
		}
	    }
	    $n++;
	    if($verb) { print STDERR "seencolor=$seencolor\n";}
	}
    }

    if($verb) {
	print STDERR "Sorted Rcoords:\n";
	for(my $i=1;$i<=$rcontigno;$i++) {
	    for(my $j=0;$j<=$rcoordno[$i];$j++) {
		print STDERR "$i $j ",$srcoord[$i][$j]," ",$rcoord[$i][$srcoord[$i][$j]][0]," ",$rcoord[$i][$srcoord[$i][$j]][1]," ",$rcoord[$i][$srcoord[$i][$j]][2]," ",$rcoord[$i][$srcoord[$i][$j]][3]," ",$rcoord[$i][$srcoord[$i][$j]][4]," ",$rcoord[$i][$srcoord[$i][$j]][5]," ",$rcoord[$i][$srcoord[$i][$j]][6],"\n";
	    }
	}
	print STDERR "Sorted Qcoords:\n";
	for(my $i=1;$i<=$qcontigno;$i++) {
	    for(my $j=0;$j<=$qcoordno[$i];$j++) {
		print STDERR "$i $j ",$sqcoord[$i][$j]," ",$qcoord[$i][$sqcoord[$i][$j]][0]," ",$qcoord[$i][$sqcoord[$i][$j]][1]," ",$qcoord[$i][$sqcoord[$i][$j]][2]," ",$qcoord[$i][$sqcoord[$i][$j]][3]," ",$qcoord[$i][$sqcoord[$i][$j]][4]," ",$qcoord[$i][$sqcoord[$i][$j]][5]," ",$qcoord[$i][$sqcoord[$i][$j]][6],"\n";
	    }
	}
    }


    if($verb) { print STDERR "Parsing genes in rcoords:\n";}
    for(my $i=1;$i<=$rcontigno;$i++) {
	my $n;
	for($n=0;$n<=$rcoordno[$i];$n++) {
	    if($rcoord[$i][$srcoord[$i][$n]][3]==1 && $rcoord[$i][$srcoord[$i][$n]][1]==0) { # start of gene
		my $genecolor=$rcoord[$i][$srcoord[$i][$n]][6];
		my $end_gene=$srcoord[$i][$n]+1;
		if($verb) { print STDERR "Gene: $i $n ",$srcoord[$i][$n]," $end_gene\n";}
		my $kg=$n+1;
		my $ind=0;
		my $newcolor=0;
		my @endlist=();
		if($end_gene==$srcoord[$i][$kg]) { # gene is contained in alignment 
		    my $color1=-2;
		    my $color2=-2;
		    my $i1=$n-1;
		    my $i2=$n+2;
		    
		    while($i1>=0 && $rcoord[$i][$srcoord[$i][$i1]][3]) { $i1--; }
		    if($i1>=0) { $color1=$rcoord[$i][$srcoord[$i][$i1]][6];}
		    
		    while($i2<=$rcoordno[$i] && $rcoord[$i][$srcoord[$i][$i2]][3]) { $i2++;} 
		    if($i2<=$rcoordno[$i]) { 
			$color2=$rcoord[$i][$srcoord[$i][$i2]][6];
		    }

		    # test if gene is within a colored region
		    if($color1==-2 && $color2>=-1 && $color2 != $genecolor) { 
			if($genecolor) {
			    $newcolor=$color2;
			    $ind=1;
			}
		    }
		    elsif($color2==-2 && $color1>=-1 && $color1 != $genecolor) {
			if($genecolor) {
			    $newcolor=$color1;
			    $ind=1;
			}
		    }
		    elsif($color1>=-1 && $color2>=-1 && $color1 != $genecolor) {
			if($genecolor && $color1!=$color2) { print STDERR "Error: genes not contained within same colored region: color1=$color1 color2=$color2! $i1 $i2\n"; exit;}
			if(($genecolor && $color1>=-1) || (!$genecolor && ($color1==$color2) && $color1>-1)) {
			    $newcolor=$color1;
			    $ind=1;
			}
		    }
		    
		}
		else {
		    push(@endlist,$end_gene);
		}

		while($#endlist>-1) {
		    if($rcoord[$i][$srcoord[$i][$kg]][6]!=$genecolor) {
			$ind=1;
			if($newcolor!=-1) {
			    if($newcolor != $rcoord[$i][$srcoord[$i][$kg]][6] && $newcolor>$color && $rcoord[$i][$srcoord[$i][$kg]][6]>$color) { $newcolor=-1;}
			    else { $newcolor=$rcoord[$i][$srcoord[$i][$kg]][6]; }
			}
		    }
		    
		    # remove $srcoord[$i][$kg] from @endlist if present
		    my $z=0;
		    while($z<=$#endlist) {
			if($srcoord[$i][$kg]==$endlist[$z]) {
			    $endlist[$z]=$endlist[$#endlist];
			    $z=$#endlist;
			    $#endlist--;
			}
			$z++;
		    }
		    
		    if($rcoord[$i][$srcoord[$i][$kg]][1]==0) {
			push(@endlist,$srcoord[$i][$kg]+1);
		    }

		    if($#endlist>-1) { $kg++;}
		}

#	    while($srcoord[$i][$kg]!=$end_gene) { 
#		if($rcoord[$i][$srcoord[$i][$kg]][6]!=$genecolor) {
#		    $ind=1;
#		    if($newcolor!=-1) {
#			if($newcolor != $rcoord[$i][$srcoord[$i][$kg]][6] && $newcolor>$color && $rcoord[$i][$srcoord[$i][$kg]][6]>$color) { $newcolor=-1;}
#			else { $newcolor=$rcoord[$i][$srcoord[$i][$kg]][6]; }
#		    }
#		}
#		$kg++;
#	    }


		if($ind) {
		    if($verb) { print STDERR "Mark gene with $newcolor from $n to $kg\n";}
		    for(my $j=$n; $j<=$kg;$j++) { 
			$rcoord[$i][$srcoord[$i][$j]][6]=$newcolor;
			if(!$rcoord[$i][$srcoord[$i][$j]][3]) { # alignment
			    $qcoord[$rcoord[$i][$srcoord[$i][$j]][4]][$rcoord[$i][$srcoord[$i][$j]][5]][6]=$newcolor;
			}
		    }
		}
	    }
	}

	$n=$rcoordno[$i];
	while($n>=0) { # propagate -1's to the beginning
	    if($rcoord[$i][$srcoord[$i][$n]][1] && $rcoord[$i][$srcoord[$i][$n]][6]==-1) { # found a -1 end
		my @minend;
		my $ends=0;
		do {
		    if($rcoord[$i][$srcoord[$i][$n]][1]) {
			$minend[$srcoord[$i][$n]-1]=1;
			$ends++;
		    }
		    else {
			if($minend[$srcoord[$i][$n]]) {
			    $ends--;
			}
		    }
		    $rcoord[$i][$srcoord[$i][$n]][6]=-1;
		    $n--;
		} while ($ends);
	    }
	    else { $n--; }
	}
    }



    if($verb) { print STDERR "Parsing genes in qcoords:\n";}
    for(my $i=1;$i<=$qcontigno;$i++) {
	my $n;
	for($n=0;$n<=$qcoordno[$i];$n++) {
	    if($qcoord[$i][$sqcoord[$i][$n]][3]==1 && $qcoord[$i][$sqcoord[$i][$n]][1]==0) { # start of gene
		my $genecolor=$qcoord[$i][$sqcoord[$i][$n]][6];
		my $end_gene=$sqcoord[$i][$n]+1;
		if($verb) { print STDERR "Geneq: $i $n ",$sqcoord[$i][$n]," $end_gene\n";}
		my $kg=$n+1;
		my $ind=0;
		my $newcolor=0;
		my @endlist=();
		if($end_gene==$sqcoord[$i][$kg]) { # gene is contained in alignment 
		    my $color1=-2;
		    my $color2=-2;
		    my $i1=$n-1;
		    my $i2=$n+2;

		    while($i1>=0 && $qcoord[$i][$sqcoord[$i][$i1]][3]) { $i1--; }
		    if($i1>=0) { $color1=$qcoord[$i][$sqcoord[$i][$i1]][6];}

		    while($i2<=$qcoordno[$i] && $qcoord[$i][$sqcoord[$i][$i2]][3]) { $i2++;} 
		    if($i2<=$qcoordno[$i]) { 
			$color2=$qcoord[$i][$sqcoord[$i][$i2]][6];
		    }

		    # test if gene is within a colored region
		    if($color1==-2 && $color2>=-1 && $color2 != $genecolor) { 
			if($genecolor) {
			    $newcolor=$color2;
			    $ind=1;
			}
		    }
		    elsif($color2==-2 && $color1>=-1 && $color1 != $genecolor) {
			if($genecolor) {
			    $newcolor=$color1;
			    $ind=1;
			}
		    }
		    elsif($color1>=-1 && $color2>=-1 && $color1 != $genecolor) {
			if($genecolor && $color1!=$color2) { print STDERR "Error: genes not contained within same colored region: color1=$color1 color2=$color2!\n"; exit;}
			if(($genecolor && $color1>=-1) || (!$genecolor && ($color1==$color2) && $color1>-1)) {
			    $newcolor=$color1;
			    $ind=1;
			}
		    }
		    
		}
		else {
		    push(@endlist,$end_gene);
		}

		while($#endlist>-1) {
		    if($qcoord[$i][$sqcoord[$i][$kg]][6]!=$genecolor) {
			$ind=1;
			if($newcolor!=-1) {
			    if($newcolor != $qcoord[$i][$sqcoord[$i][$kg]][6] && $newcolor>$color && $qcoord[$i][$sqcoord[$i][$kg]][6]>$color) { $newcolor=-1;}
			    else { $newcolor=$qcoord[$i][$sqcoord[$i][$kg]][6]; }
			}
		    }

		    # remove $sqcoord[$i][$kg] from @endlist if present
		    my $z=0;
		    while($z<=$#endlist) {
			if($sqcoord[$i][$kg]==$endlist[$z]) {
			    $endlist[$z]=$endlist[$#endlist];
			    $z=$#endlist;
			    $#endlist--;
			}
			$z++;
		    }

		    if($qcoord[$i][$sqcoord[$i][$kg]][1]==0) {
			push(@endlist,$sqcoord[$i][$kg]+1);
		    }

		    if($#endlist>-1) { $kg++;}
		}

#	    while($sqcoord[$i][$kg]!=$end_gene) { 
#		if($qcoord[$i][$sqcoord[$i][$kg]][6]!=$genecolor) {
#		    $ind=1;
#		    if($newcolor!=-1) {
#			if($newcolor != $qcoord[$i][$sqcoord[$i][$kg]][6] && $newcolor>$color && $qcoord[$i][$sqcoord[$i][$kg]][6]>$color) { $newcolor=-1;}
#			else { $newcolor=$qcoord[$i][$sqcoord[$i][$kg]][6]; }
#		    }
#		}
#		$kg++;
#	    }

		if($ind) {
		    if($verb) { print STDERR "Mark geneq with $newcolor from $n to $kg\n";}
		    for(my $j=$n; $j<=$kg;$j++) { 
			$qcoord[$i][$sqcoord[$i][$j]][6]=$newcolor;
		    }
		}
	    }
	}
    }

    if($verb) {
	print STDERR "Sorted Rcoords:\n";
	for(my $i=1;$i<=$rcontigno;$i++) {
	    for(my $j=0;$j<=$rcoordno[$i];$j++) {
		print STDERR "$i $j ",$srcoord[$i][$j]," ",$rcoord[$i][$srcoord[$i][$j]][0]," ",$rcoord[$i][$srcoord[$i][$j]][1]," ",$rcoord[$i][$srcoord[$i][$j]][2]," ",$rcoord[$i][$srcoord[$i][$j]][3]," ",$rcoord[$i][$srcoord[$i][$j]][4]," ",$rcoord[$i][$srcoord[$i][$j]][5]," ",$rcoord[$i][$srcoord[$i][$j]][6],"\n";
	    }
	}
	print STDERR "Sorted Qcoords:\n";
	for(my $i=1;$i<=$qcontigno;$i++) {
	    for(my $j=0;$j<=$qcoordno[$i];$j++) {
		print STDERR "$i $j ",$sqcoord[$i][$j]," ",$qcoord[$i][$sqcoord[$i][$j]][0]," ",$qcoord[$i][$sqcoord[$i][$j]][1]," ",$qcoord[$i][$sqcoord[$i][$j]][2]," ",$qcoord[$i][$sqcoord[$i][$j]][3]," ",$qcoord[$i][$sqcoord[$i][$j]][4]," ",$qcoord[$i][$sqcoord[$i][$j]][5]," ",$qcoord[$i][$sqcoord[$i][$j]][6],"\n";
	    }
	}
    }

    return($color);
}


#*********************************
# print found conserved regions 
#*********************************
sub print_regions {
    my($conflict)=@_;

    my $lastcolor=-1;
    my $contig;
    foreach $contig (keys %rcontig) {

	my $rno=$rcontig{$contig};
	$lastcolor=-1;

	my $i=0;
	while($i<=$rcoordno[$rno]) {

	    my $color=$rcoord[$rno][$srcoord[$rno][$i]][6];

	    if($color==-1 || $color==0) {  # conflicting region or not covered gene
		$lastcolor=-1;
		while($rcoord[$rno][$srcoord[$rno][$i]][6]<=0 && $i<=$rcoordno[$rno]) {
		    $i++;
		}
	    }
	    elsif($color != $lastcolor) { # start of non-conflicting region
		$lastcolor=$color;
		my $rsminno=$i;
		my $qno=0; 
		my $qinitcoordno=-1;
		my $rdir=$rcoord[$rno][$srcoord[$rno][$i]][2];
		my $start_alignm=0;
		my $start_gene_cov=0;
		my $rgene_len=0;
		my $rgene_cov=0;
		my $ralignm_len=0;
		my $rpid=0;
		my $rmaxid=0;
		my @e;
		my $rnoex=0;
		my $rnogene=0;

		my ($nextexno,$last_al_cov,$start_alignm_coord,$rsmaxno);
		while($rcoord[$rno][$srcoord[$rno][$i]][6]==$color && $i<=$rcoordno[$rno]) {

		    if($verb) { print STDERR "Alignm: $rno ",$srcoord[$rno][$i]," ",$rcoord[$rno][$srcoord[$rno][$i]][0]," ",$rcoord[$rno][$srcoord[$rno][$i]][1]," ",$rcoord[$rno][$srcoord[$rno][$i]][2]," ",$rcoord[$rno][$srcoord[$rno][$i]][3]," ",$rcoord[$rno][$srcoord[$rno][$i]][4]," ",$rcoord[$rno][$srcoord[$rno][$i]][5]," ",$rcoord[$rno][$srcoord[$rno][$i]][6],"\n";}

		    if($rcoord[$rno][$srcoord[$rno][$i]][3]) { # gene
			if($rcoord[$rno][$srcoord[$rno][$i]][1]) { # end of gene
			    $start_gene_cov=0;
			    if($start_alignm && $nextexno<=$#e) { 
				while($nextexno<=$#e) {
				    if($nextexno%2) {
					$rgene_cov+=$e[$nextexno]-$last_al_cov+1;
				    }
				    else {
					$last_al_cov=$e[$nextexno];
					if($verb) { print STDERR "1 last_al_cov=$last_al_cov\n";}
				    }
				    $nextexno++;
				}
				if($verb) { print STDERR "1 rgene_cov = $rgene_cov ; last + = ",$e[$#e]-$last_al_cov+1," from $last_al_cov to ",$e[$#e],"\n";}
			    }
			}
			else { # start of gene
			    $last_al_cov=0;
			    @e=();
			    @e=split(/\s+/,$rcoord[$rno][$srcoord[$rno][$i]][4]);
			    for(my $k=0;$k<=$#e;$k+=2) {
				$rgene_len+=$e[$k+1]-$e[$k]+1;
			    }
			    $rnoex+=($#e+1)/2;
			    $rnogene++;
			    $nextexno=0;
			    if($start_alignm) {
				$last_al_cov=$e[0];
				if($verb) { print STDERR "2 last_al_cov=$last_al_cov\n";}
				$nextexno++;
			    }
			    $start_gene_cov=1;
			}
		    }
		    elsif($rcoord[$rno][$srcoord[$rno][$i]][3]==0) { # alignment

			if(!$qno) { $qno=$rcoord[$rno][$srcoord[$rno][$i]][4];}

			if($qinitcoordno<0) { 
			    $qinitcoordno=$rcoord[$rno][$srcoord[$rno][$i]][5]; 
			}

			if($rcoord[$rno][$srcoord[$rno][$i]][1]) { # end of alignment
			    $start_alignm--;
			    if(!$start_alignm) { 
				$ralignm_len+=$rcoord[$rno][$srcoord[$rno][$i]][0]-$start_alignm_coord;
				if($start_gene_cov) { # gene started and no more alignments covering gene
				    while($nextexno<=$#e && $e[$nextexno]<=$rcoord[$rno][$srcoord[$rno][$i]][0]) {
					if($nextexno%2) {
					    $rgene_cov+=$e[$nextexno]-$last_al_cov+1;
					    if($verb) { print STDERR "2 rgene_cov = $rgene_cov ; last + = ",$e[$nextexno]-$last_al_cov+1," from $last_al_cov to ",$e[$nextexno],"\n";}
					}
					else {
					    $last_al_cov=$e[$nextexno];
					    if($verb) { print STDERR "3 last_al_cov=$last_al_cov\n";}
					}
					$nextexno++;

				    }
				    if($nextexno%2 && $last_al_cov<=$rcoord[$rno][$srcoord[$rno][$i]][0]) { 
					$rgene_cov+=$rcoord[$rno][$srcoord[$rno][$i]][0]-$last_al_cov+1;
					if($verb) { print STDERR "3 rgene_cov = $rgene_cov ; last + = ",$rcoord[$rno][$srcoord[$rno][$i]][0]-$last_al_cov+1," from $last_al_cov to ",$rcoord[$rno][$srcoord[$rno][$i]][0],"\n";}
				    }
				}
			    }
			}
			else { # start of alignment
			    $start_alignm++;
			    if($start_alignm==1) { 
				$start_alignm_coord=$rcoord[$rno][$srcoord[$rno][$i]][0];
				if($start_gene_cov) { # only one gene started
				    while($nextexno<=$#e && $e[$nextexno]<$rcoord[$rno][$srcoord[$rno][$i]][0]) { $nextexno++;}
				    if($nextexno<=$#e) {
					if($nextexno%2) {
					    if($last_al_cov<$rcoord[$rno][$srcoord[$rno][$i]][0]) { 
						$last_al_cov=$rcoord[$rno][$srcoord[$rno][$i]][0];
						if($verb) { print STDERR "4 last_al_cov=$last_al_cov\n";}
					    }
					}
					else { 
					    $last_al_cov=$e[$nextexno]; 
					    if($verb) { print STDERR "5 last_al_cov=$last_al_cov\n";}
					    $nextexno++;
					}
				    }
				}
			    }
			}
		    }
		    $rsmaxno=$i;
		    $i++;
		}

		# determine qcoord region ; take care of unmarked genes
		my ($qcntg,$qinit,$qsminno,$qdir,$qgene_len,$qgene_cov,$qalignm_len);

		$qcntg=qcontigname($qno);
		$qinit=sqcoordno($qno,$qinitcoordno);
		$qsminno=$qinit;
		
		while($qsminno>=0 && $qcoord[$qno][$sqcoord[$qno][$qsminno]][6]==$color) { $qsminno--;}
		$qsminno++;
		
		$qdir=$qcoord[$qno][$sqcoord[$qno][$qsminno]][2];
		$start_alignm=0;
		$start_gene_cov=0;
		$qgene_len=0;
		$qgene_cov=0;
		$qalignm_len=0;
		
		my $j=$qsminno;
		my $qnoex=0;
		my $qnogene=0;
		while($qcoord[$qno][$sqcoord[$qno][$j]][6]==$color && $j<=$qcoordno[$qno]) {

		    if($qcoord[$qno][$sqcoord[$qno][$j]][3]) { # gene
			if($qcoord[$qno][$sqcoord[$qno][$j]][1]) { # end of gene
			    $start_gene_cov=0;
			    if($start_alignm && $nextexno<=$#e) { 
				while($nextexno<=$#e) {
				    if($nextexno%2) {
					$qgene_cov+=$e[$nextexno]-$last_al_cov+1;
				    }
				    else {
					$last_al_cov=$e[$nextexno];
					if($verb) { print STDERR "1q last_al_cov=$last_al_cov\n";}
				    }
				    $nextexno++;
				}
				if($verb) { print STDERR "4 qgene_cov = $qgene_cov ; last + = ",$e[$#e]-$last_al_cov+1," from $last_al_cov to ",$e[$#e],"\n";}
			    }
			}
			else { # start of gene
			    $last_al_cov=0;
			    @e=();
			    @e=split(/\s+/,$qcoord[$qno][$sqcoord[$qno][$j]][4]);
			    for(my $k=0;$k<=$#e;$k+=2) {
				$qgene_len+=$e[$k+1]-$e[$k]+1;
			    }
			    $qnoex+=($#e+1)/2;
			    $qnogene++;
			    $nextexno=0;
			    if($start_alignm) {
				$last_al_cov=$e[0];
				if($verb) { print STDERR "2q last_al_cov=$last_al_cov\n";}
				$nextexno++;
			    }
			    $start_gene_cov=1;
			}
		    }
		    elsif($qcoord[$qno][$sqcoord[$qno][$j]][3]==0) { # alignment
			if($qcoord[$qno][$sqcoord[$qno][$j]][1]) { # end of alignment
			    $start_alignm--;
			    if(!$start_alignm) { 
				$qalignm_len+=$qcoord[$qno][$sqcoord[$qno][$j]][0]-$start_alignm_coord;
				if($start_gene_cov) { # gene started and no more alignments covering gene
				    while($nextexno<=$#e && $e[$nextexno]<=$qcoord[$qno][$sqcoord[$qno][$j]][0]) {
					if($nextexno%2) {
					    $qgene_cov+=$e[$nextexno]-$last_al_cov+1;
					    if($verb) { print STDERR "5 qgene_cov = $qgene_cov ; last + = ",$e[$nextexno]-$last_al_cov+1," from $last_al_cov to ",$e[$nextexno],"\n";}
					}
					else {
					    $last_al_cov=$e[$nextexno];
					    if($verb) { print STDERR "3q last_al_cov=$last_al_cov\n";}
					}
					$nextexno++;
				    }
				    if($nextexno%2 && $last_al_cov<=$qcoord[$qno][$sqcoord[$qno][$j]][0]) { 
					$qgene_cov+=$qcoord[$qno][$sqcoord[$qno][$j]][0]-$last_al_cov+1;
					if($verb) { print STDERR "6 qgene_cov = $qgene_cov ; last + = ",$qcoord[$qno][$sqcoord[$qno][$j]][0]-$last_al_cov+1," from $last_al_cov to ",$qcoord[$qno][$sqcoord[$qno][$j]][0],"\n";}
				    }
				}
			    }
			}
			else { # start of alignment
			    $start_alignm++;
			    if($start_alignm==1) { 
				$start_alignm_coord=$qcoord[$qno][$sqcoord[$qno][$j]][0];
				if($start_gene_cov) { # only one alignment started
				    while($nextexno<=$#e && $e[$nextexno]<$qcoord[$qno][$sqcoord[$qno][$j]][0]) { $nextexno++;}
				    if($nextexno<=$#e) {
					if($nextexno%2) {
					    if($last_al_cov<$qcoord[$qno][$sqcoord[$qno][$j]][0]) { 
						$last_al_cov=$qcoord[$qno][$sqcoord[$qno][$j]][0];
						if($verb) { print STDERR "4q last_al_cov=$last_al_cov\n";}
					    }
					}
					else { 
					    $last_al_cov=$e[$nextexno]; 
					    if($verb) { print STDERR "5q last_al_cov=$last_al_cov\n";}
					    $nextexno++;
					}
				    }
				}
			    }
			}
		    }
		    $j++;
		}
		my $qsmaxno=$j-1;

		# print region
		my $coord;
		if($rsminno == 0) {
		    $coord=1;
		}
		else {
		    $coord=$rcoord[$rno][$srcoord[$rno][$rsminno-1]][0];
		}
		my $rstartcoord = $coord < $rcoord[$rno][$srcoord[$rno][$rsminno]][0]-$flankreg ? $rcoord[$rno][$srcoord[$rno][$rsminno]][0]-$flankreg : $coord;
		my $rendcoord;
		if($rsmaxno == $rcoordno[$rno]) {
		    $rendcoord = $flankreg + $rcoord[$rno][$srcoord[$rno][$rsmaxno]][0];
		}
		else {
		    $rendcoord = $flankreg + $rcoord[$rno][$srcoord[$rno][$rsmaxno]][0] < $rcoord[$rno][$srcoord[$rno][$rsmaxno+1]][0] ? $flankreg + $rcoord[$rno][$srcoord[$rno][$rsmaxno]][0] : $rcoord[$rno][$srcoord[$rno][$rsmaxno+1]][0];
		}
		if($rendcoord>$rlcontig[$rno]) { $rendcoord=$rlcontig[$rno];}
		if($rdir==-1) {
		    printf("%10d %10d | ",$rendcoord,$rstartcoord);
		}
		else {
		    printf("%10d %10d | ",$rstartcoord,$rendcoord);
		}
		if($qsminno == 0) {
		    $coord=1;
		}
		else {
		    $coord=$qcoord[$qno][$sqcoord[$qno][$qsminno-1]][0];
		}
		my $qstartcoord = $coord < $qcoord[$qno][$sqcoord[$qno][$qsminno]][0]-$flankreg ? $qcoord[$qno][$sqcoord[$qno][$qsminno]][0]-$flankreg : $coord;
		my $qendcoord;
		if($qsmaxno == $qcoordno[$qno]) {
		    $qendcoord = $flankreg + $qcoord[$qno][$sqcoord[$qno][$qsmaxno]][0];
		}
		else {
		    $qendcoord = $flankreg + $qcoord[$qno][$sqcoord[$qno][$qsmaxno]][0] < $qcoord[$qno][$sqcoord[$qno][$qsmaxno+1]][0] ? $flankreg + $qcoord[$qno][$sqcoord[$qno][$qsmaxno]][0] : $qcoord[$qno][$sqcoord[$qno][$qsmaxno+1]][0];
		}
		if($qendcoord>$qlcontig[$qno]) { $qendcoord=$qlcontig[$qno];}
		if($qdir==-1) {
		    printf("%10d %10d | ",$qendcoord,$qstartcoord);
		}
		else {
		    printf("%10d %10d | ", $qstartcoord,$qendcoord);
		}
	   
		my ($rcoverage,$qcoverage);

		if($rgene_len) { 
		    $rcoverage=$rgene_cov*100/$rgene_len;
		}
		else {
		    $rcoverage=0;
		}
		
		if($qgene_len) { 
		    $qcoverage=$qgene_cov*100/$qgene_len;
		}
		else {
		    $qcoverage=0;
		}
	    
	    
		
		printf("%9d %9d %5d %5d %9d( %6.2f\%) | %9d %9d %5d %5d %9d( %6.2f\%) |%s|",$ralignm_len,$rgene_len,$rnoex,$rnogene,$rgene_cov,$rcoverage,$qalignm_len,$qgene_len,$qnoex,$qnogene,$qgene_cov,$qcoverage,$conflict);
		print " $contig $qcntg\n";
		
	    }
	}
    }
}

#*********************************
# check if alignment in rcoord has color 
#*********************************
sub checkr {
    my($contigno,$start,$end,$color)=@_;

    my $retval=0;

    my $i=$start;

    while($i<=$end && !$retval) {
	if($rcoord[$contigno][$srcoord[$contigno][$i]][6]==$color) { return 1;}
	$i++;
    }

    return $retval;
}

#*********************************
# check if alignment in qcoord has color 
#*********************************
sub checkq {
    my($contigno,$start,$end,$color)=@_;

    my $retval=0;

    my $i=$start;

    while($i<=$end && !$retval) {
	if($qcoord[$contigno][$sqcoord[$contigno][$i]][6]==$color) { return 1;}
	$i++;
    }

    return $retval;
}


#*********************************
# mark region with given color starting from rcoord and linking to qcoord
#*********************************
sub markr {
    my($contigno,$start,$end,$color)=@_;

    if($verb) { print STDERR "Markr $contigno with $color from $start to $end\n";}

    if($color>-1) {
	my $i=$start;
	while($i<=$end && $color>-1) {
	    my $prevcolor=$rcoord[$contigno][$srcoord[$contigno][$i]][6];
	    if($prevcolor == 0 || $prevcolor == $color) { # ok to mark
		$rcoord[$contigno][$srcoord[$contigno][$i]][6]=$color;
		if($rcoord[$contigno][$srcoord[$contigno][$i]][3]!=1) {
		    $prevcolor=$qcoord[$rcoord[$contigno][$srcoord[$contigno][$i]][4]][$rcoord[$contigno][$srcoord[$contigno][$i]][5]][6];
		    if($prevcolor == 0 || $prevcolor == $color) { # ok to mark
			$qcoord[$rcoord[$contigno][$srcoord[$contigno][$i]][4]][$rcoord[$contigno][$srcoord[$contigno][$i]][5]][6]=$color;
		    }
		    else { $color=-1;}
		}
	    }
	    else {$color=-1;}
	    $i++;
	}
    }
    if($color==-1) {
	my $i=$end;
	my @minend;
	my $ends=0;
	do {
	    if($rcoord[$contigno][$srcoord[$contigno][$i]][1]) {
		$minend[$srcoord[$contigno][$i]-1]=1;
		$ends++;
	    }
	    else {
		if($minend[$srcoord[$contigno][$i]]) {
		    $ends--;
		}
	    }
	    $rcoord[$contigno][$srcoord[$contigno][$i]][6]=-1;
	    $i--;
	} while ($ends);

#	for(my $i=$start;$i<=$end;$i++) {
#	    $rcoord[$contigno][$srcoord[$contigno][$i]][6]=-1;
#	    if($rcoord[$contigno][$srcoord[$contigno][$i]][3] !=1) { 
#		$qcoord[$rcoord[$contigno][$srcoord[$contigno][$i]][4]][$rcoord[$contigno][$srcoord[$contigno][$i]][5]][6]=-1;
#	    }
#	}
    }
}


#*********************************
# mark region with given color starting from qcoord and linking to rcoord
#*********************************
sub markq {
    my($contigno,$start,$end,$color,$okcolor)=@_;

    if($verb) { print STDERR "Markq $contigno with $color from $start to $end, where okcolor=$okcolor\n";}

    if($color>-1) {
	my $i=$start;
	while($i<=$end && $color>-1) {
	    my $prevcolor=$qcoord[$contigno][$sqcoord[$contigno][$i]][6];
	    if($color<=$okcolor && $prevcolor!=0 && $prevcolor!=$color) { $color=-1;}
	    elsif($prevcolor!=-1 && ($prevcolor == 0 || $prevcolor == $color || $prevcolor<=$okcolor)) { # ok to mark
		$qcoord[$contigno][$sqcoord[$contigno][$i]][6]=$color;
		if($qcoord[$contigno][$sqcoord[$contigno][$i]][3]!=1) { # alignment
		    $prevcolor=$rcoord[$qcoord[$contigno][$sqcoord[$contigno][$i]][4]][$qcoord[$contigno][$sqcoord[$contigno][$i]][5]][6];
		    if($prevcolor!=-1 && ($prevcolor == 0 || $prevcolor == $color || $prevcolor <= $okcolor)) { # ok to mark
			$rcoord[$qcoord[$contigno][$sqcoord[$contigno][$i]][4]][$qcoord[$contigno][$sqcoord[$contigno][$i]][5]][6]=$color;
		    }
		    else { $color=-1;}
		}
	    }
	    else {$color=-1;}
	    $i++;
	}
    }
    if($color==-1) {
	my $i=$end;
	my @minend;
	my $ends=0;
	do {
	    if($qcoord[$contigno][$sqcoord[$contigno][$i]][1]) {
		$minend[$sqcoord[$contigno][$i]-1]=1;
		$ends++;
	    }
	    else {
		if($minend[$sqcoord[$contigno][$i]]) {
		    $ends--;
		}
	    }
	    $qcoord[$contigno][$sqcoord[$contigno][$i]][6]=-1;
	    if($qcoord[$contigno][$sqcoord[$contigno][$i]][3] != 1 ) {
		$rcoord[$qcoord[$contigno][$sqcoord[$contigno][$i]][4]][$qcoord[$contigno][$sqcoord[$contigno][$i]][5]][6]=-1;

	    }
	    $i--;
	} while ($ends);

#	for(my $i=$start;$i<=$end;$i++) {
#	    $qcoord[$contigno][$sqcoord[$contigno][$i]][6]=-1;
#	    if($qcoord[$contigno][$sqcoord[$contigno][$i]][3] != 1 ) {
#		$rcoord[$qcoord[$contigno][$sqcoord[$contigno][$i]][4]][$qcoord[$contigno][$sqcoord[$contigno][$i]][5]][6]=-1;
#	    }
#	}
    }
}
    

#*********************************
# check if the regions conflicts with the given alignment in rcoord
#*********************************
sub conflictr {
    my($rgsign,$rglink,$rgcoord,$asign,$alink,$acoord)=@_;

    if($rglink==0) { return 0; }
    
    if(($rgsign !=$asign) || ($rglink!=$alink)) { return 1;}

    if($qcoord[$rglink][$rgcoord][2] != $qcoord[$alink][$acoord][2]) { return 1;}

    return 0;
}

#*********************************
# check if the regions conflicts with the given alignment in qcoord
#*********************************
sub conflictq {
    my($rgsign,$rglink,$rgcoord,$asign,$alink,$acoord)=@_;

    if($rglink==0) { return 0; }
    
    if(($rgsign !=$asign) || ($rglink!=$alink)) { return 1;}

    if($rcoord[$rglink][$rgcoord][2] != $rcoord[$alink][$acoord][2]) { return(1);}

    return 0;
}


#*********************************
# returns the sqcoord for a given contigno and qcoordno
#*********************************
sub sqcoordno {
    my ($qcontigno,$qcrdno)=@_;
    

    my $min=0;
    my $max=$qcoordno[$qcontigno];

    while($min<$max) {
	my $mid=$min+int(($max-$min+1)/2);
	if($sqcoord[$qcontigno][$mid] == $qcrdno) { return $mid;}
	elsif($qcoord[$qcontigno][$sqcoord[$qcontigno][$mid]][0] < $qcoord[$qcontigno][$qcrdno][0]) { 
	    $min=$mid+1;
	}
	elsif($qcoord[$qcontigno][$sqcoord[$qcontigno][$mid]][0] > $qcoord[$qcontigno][$qcrdno][0]) { 
	    $max=$mid-1;
	}
	else {
	    $min=$mid-1;
	    while($min>=0 && $qcoord[$qcontigno][$sqcoord[$qcontigno][$min]][0] == $qcoord[$qcontigno][$qcrdno][0]) {
		if($sqcoord[$qcontigno][$min] == $qcrdno) { return $min;}
		$min--;
	    }
	    $max=$mid+1;
	    while($max<=$qcoordno[$qcontigno] && $qcoord[$qcontigno][$sqcoord[$qcontigno][$max]][0] == $qcoord[$qcontigno][$qcrdno][0]) {
		if($sqcoord[$qcontigno][$max] == $qcrdno) { return $max;}
		$max++;
	    }
	}
    }

    if($sqcoord[$qcontigno][$min]!=$qcrdno) {
	print STDERR "ERROR: Querry alignment not found: $qcontigno $qcrdno!\n";
	return(-1);
    }
    else {
	return $min;
    }
}
    

#*********************************
# returns qcontig name for a given contigno
#*********************************
sub qcontigname {
    my ($contigno)=@_;

    my $contig;
    foreach $contig (keys %qcontig) {
	if($qcontig{$contig}==$contigno) { return $contig;}
    }

    $contig="error";
    return $contig;
}



#*********************************
# read positions of the indices of interest in input file
#*********************************
sub read_ind_positions {
    my ($filename,$pos)=@_;

    open(F,$filename);
    <F>;
    <F>;
    <F>;
    $_=<F>;

    my @a=(m/\s+(\||\[.+?\])/g) ; 
    

    my ($ind_s1,$ind_e1,$ind_s2,$ind_e2,$ind_pidy,$ind_psim,$ind_lenref,$ind_lenque);

    for (my $ind=0;$ind<=$#a;$ind++){
	if ($a[$ind] eq "[S1]") {
	    $ind_s1=$ind;     
	}
	elsif ($a[$ind] eq "[E1]"){
	    $ind_e1=$ind;    
	}
	elsif($a[$ind] eq "[S2]") {
	    $ind_s2=$ind;     
	}
	elsif ($a[$ind] eq "[E2]"){
	    $ind_e2=$ind;    
	}
	elsif ($a[$ind] eq "[% IDY]"){
	    $ind_pidy=$ind;  
	}
	elsif ($a[$ind] eq "[% SIM]"){
	    $ind_psim=$ind;  
	}
	elsif ($a[$ind] eq "[LEN R]"){
	    $ind_lenref=$ind;   
	}
	elsif($a[$ind] eq "[LEN Q]") {
	    $ind_lenque=$ind;
	}
    }

    close(F);
    return($ind_s1,$ind_e1,$ind_s2,$ind_e2,$ind_pidy,$ind_psim,$ind_lenref,$ind_lenque);
}


