#!/usr/bin/perl

use strict;
use Getopt::Std;
use TempFilename;

my $usage = q/
  Run Oasis module.
  Usage:
    runoasis.pl <roseoutput file> <reference genome fasta file> <query genome fasta file> 
                <reference training file> <query training file> 
                <reference output file> <query output file> [optional_parameters]
    
    [optional_parameters]

    -d dir       directory for runing oasis (default: is oasis\/)
    -p dir       promer\/nucmer directory
    -l logfile   file to store aditional info
    -m minamtch  for promer, set the minimum length of a single match, 
                 measured in amino acids (default 3)
    -c cluster   for promer, sets the minimum length of a cluster of matches, 
                 measured in amino acids (default 3)
    
/;

die $usage if (@ARGV<7);

# get options
{
    my @args;
    while ($ARGV[0] && $ARGV[0]!~/^\-/) {
         push(@args, shift(@ARGV)); }
    getopts('d:p:l:m:c:') || die $usage;
    push(@ARGV, @args); 
}

my ($rosefile,$reffile,$quefile,$reftrain,$quetrain,$refout,$queout)=@ARGV;


my ($refname,$quename);

{
    my @temp=split(/\//,$reffile);
    $refname=$temp[$#temp];
    $refname =~ s/(.+)\.fasta$/$1/ ;
    @temp=split(/\//,$quefile);
    $quename=$temp[$#temp];
    $quename =~ s/(.+)\.fasta$/$1/ ;
}

my $oasisdir=$Getopt::Std::opt_d || "oasis/";
my $promerdir=$Getopt::Std::opt_p;
my $logfile=$Getopt::Std::opt_l;
my $minmatch=$Getopt::Std::opt_m || 3;
my $cluster=$Getopt::Std::opt_c || 3;



open(L,">$logfile") if ($logfile);

my $rgenes="reference.temp.gff";
my $qgenes="query.temp.gff";


# first remember the contigs' addresses

my (%refaddr,%queaddr);

$/=">";

storeaddr($reffile,\%refaddr);
storeaddr($quefile,\%queaddr);

$/="\n";


open(R,$reffile);
open(Q,$quefile);

open(OR,">$refout");
open(OQ,">$queout");

my ($prevref,$prevque);
my ($refseqname,$refseq,$queseqname,$queseq);

my $reftempfile=TempFilename::generate();   #"reference.temp";
my $quetempfile=TempFilename::generate();   #"query.temp";
my $temppromer=TempFilename::generate();    #"temp.promer";
my $promertempfile=TempFilename::generate();#"promer.temp";
my $oasistempfile=TempFilename::generate(); #"oasis.temp";


open(F,$rosefile);
<F>;
<F>;
while(<F>) {
    
    chomp;
    my @tag=split;

    # tag before last is reference name, and last tag is query name
    if($prevref ne $tag[-2]) { 
	$refseq=load(*R,$refaddr{$tag[-2]}); 
	$prevref=$tag[-2];
    }

    if($prevque ne $tag[-1]) {
	$queseq=load(*Q,$queaddr{$tag[-1]}); 
	$prevque=$tag[-1];
    }


    # extract temp files for the synthenic regions
    extract_region($tag[0],$tag[1],$prevref,$refseq,$reftempfile); # tag[0] and tag[1] are the coordinates of the region on the reference file
    extract_region($tag[3],$tag[4],$prevque,$queseq,$quetempfile); # tag[2] and tag[3] are the coordinates of the region on the query file
    
    # run promer
    my $command="";
    if($promerdir) {
	$command=$promerdir."/";
    }
    
    print STDERR "Running PROMER: $prevref: ",$tag[0]," ",$tag[1],", $prevque: ",$tag[3]," ",$tag[4],"\n";
    
    $command.="promer -p $temppromer $reftempfile $quetempfile";
    my $status=system($command);
    die print STDERR "ERROR: promer exited funny: $?" unless $status ==0;

    # process the promer output to oasis necessesities

    my $smallref=loadseq("$reftempfile");
    my $smallque=loadseq("$quetempfile");

    # process promer file
    process_promer("$temppromer.delta","$promertempfile.delta",$smallref,$smallque);

    unlink <temp.promer.*>;

    # run APPAL to obtain nuc.appal file
    my $nucAppalFile=TempFilename::generate();
    $command="$oasisdir/APPAL.pl $reftempfile $quetempfile $reftrain $nucAppalFile";
    system($command);

    # run promer-to-appal.pl to obtain amino.appal file
    my $aminoAppalFile=TempFilename::generate();
    $command="$oasisdir/promer-to-appal.pl $promertempfile.delta > $aminoAppalFile";
    system($command);

    # run oasis
    $command="$oasisdir/oasis $reftempfile $quetempfile $reftrain $quetrain $promertempfile.delta $nucAppalFile $aminoAppalFile $rgenes $qgenes > $oasistempfile";
    my $start = (times)[0];
    $status=system($command);
    my $end = (times)[0];
    printf L "Oasis took %.5f CPU seconds for reference len = %d bp and query len = %d bp.\n", $end - $start,abs($tag[0]-$tag[1]+1),abs($tag[3]-$tag[4]+1) if($logfile);
    printf STDERR  "Oasis took %.5f-%.5f=%.5f CPU seconds for reference len = %d bp and query len = %d bp.\n", $start,$end,$end - $start,abs($tag[0]-$tag[1]+1),abs($tag[3]-$tag[4]+1) if($logfile);

    #die print STDERR "ERROR: oasis exited funny: $?" unless $status ==0;
    
    # for current implementation only of oasis: I need to write files $rgenes and $qgenes
    create_files($oasistempfile,$rgenes,$qgenes);

    # write gene predictions to output file with the coordinates shifted according to big contig
    write_genes($rgenes,*OR,$tag[0],$tag[1]);
    write_genes($qgenes,*OQ,$tag[3],$tag[4]);

    # delete temp files
    unlink($nucAppalFile);
    unlink($aminoAppalFile);
}

close(F);
close(R);
close(Q);
close(OR);
close(OQ);
close(L) if($logfile);


#----------------------------------------------------------
##### Subroutines #########################################
#----------------------------------------------------------

sub write_genes {
    my ($inpfile,$outpfile,$startcoord,$endcoord)=@_;

    open(I,$inpfile);
    while(<I>) {
	chomp;
	my @f=split(/\s+/);
	
	if($startcoord<$endcoord) {
	    $f[3]+=$startcoord-1;
	    $f[4]+=$startcoord-1;
	}
	else {
	    $f[3]=$startcoord-$f[3]+1;
	    $f[4]=$startcoord-$f[4]+1;
            $f[6]="-";
	}

	print $outpfile join("\t",@f),"\n";
    }
    close(I);
}



sub create_files {
    my ($oasisfile,$refgenes,$quegenes)=@_;
    
    my $write=0;

    open(G,$oasisfile);
    my $times=0;
    while(<G>) {

	if(/^\#/) { $write=0; }
	if($write) { print T $_; }

	if($_ eq "##Type DNA\n") {
	    
	    if($times) { # query genes found
		close(T);
		open(T,">$quegenes");
		$write=1;
	    }
	    else {
		$times++;
		open(T,">$refgenes");
		$write=1;
	    }
	}
	
    }
    close(T);
}


sub extract_region {
    my ($start,$end,$name,$seq,$outfile)=@_;

    my ($min,$max);

    if($start<$end) { $min=$start; $max=$end; }
    else { $min=$end; $max=$start};

    my $region=substr($seq,$min-1,$max-$min+1);
 
    # reverse sequence if on reverse strand
    if($start>$end) {
	$region=reverse $region;
	$region =~ tr/ACGTacgt/TGCAtgca/;
    }

    open(O,">$outfile");
    print O ">$name\n$region\n";
    close(O);
}
    

sub load {
    my ($file,$address)=@_;

    $/=">";
    seek $file, $address, 0;
    
    my $line=<$file>;

    $/="\n";
    
    chop($line);

    my($name, $seq);
    ($name)= ($line =~ /^(\S+)\s+/);
    my $pos=index($line,"\n");
    $seq=substr($line,$pos+1);

    $seq =~ tr/\n//d;
    $seq =~ tr/\r//d;

    return($seq);
}


sub storeaddr {
    my ($filename,$addr)=@_;
    
    open(F,$filename);
    for (my $curpos = tell F; $_ = <F>; $curpos = tell F) {
	chomp;
	if($_) {
	    my $name;
	    ($name)=/^(\S+)\s+/;
	    $$addr{$name}=$curpos;
	}
    }
    close(F);
}

sub loadseq {
    my ($seqfile)=@_;

    $/=">";   

    open(F,$seqfile);
    my($name, $seq);

    while(<F>) {
	chomp;
	if($_) {
	    #my($name,$seq)=/^(\S+)[ \t\S]*\n((\S+\n?)*)$/;exit;
	    
	    ($name)=/^(\S+)\s+/;
	    my $pos=index($_,"\n");
	    $seq=substr($_,$pos+1);
	    
	    die "ERROR 21: Wrong FASTA format: .$_." if (!$name || !$seq);
	    
	    $seq =~ tr/\n//d;
	    $seq =~ tr/\r//d;
	    
	    $seq=lc($seq);
	}
    }
    close(F);
    $/="\n";
    return($seq);
}

sub process_promer {
    my ($promertemp,$promerout,$smallref,$smallque)=@_;

    open(O,">$promerout");
    open(P,$promertemp);

    my $line=<P>;
    print O $line;
    $line=<P>;
    print O $line;
    $line=<P>;
    print O $line;

    my $rend=0;
    my $qend=0;
    while(<P>) {
	my @a=split;
	if($#a>1) { # header
	    $line=$_;
	    if(($a[0]<=$a[1]) && ($a[2]<=$a[3])) { # positive alignment
		if($a[0] <= $rend+1 || $a[2] <= $qend+1) { # continuation of the same alignment => print it
		    $rend = $rend < $a[1] ? $a[1] : $rend;
		    $qend = $qend < $a[3] ? $a[3] : $qend;
		}
		else { 
		    # first do a new promer for the uncovered region
		    my $output=promer_run($smallref,$smallque,$rend+1,$qend+1,$a[0],$a[2]);
		    print O $output;
		    $rend=$a[1];
		    $qend=$a[3];
		}
		print O $line;
		do {
		    $line=<P>;
		    print O $line;
		    chomp($line);
		} while ($line != 0);
	    }
	    else { # ignore alignment
		do {
		    $line=<P>;
		    chomp($line);
		} while($line != 0);
	    }
	}
    }
    
    # now process rest of the sequences from rend and qend
    my $rlen=length($smallref);
    my $qlen=length($smallque);
    
    if($rend+1<$rlen && $qend+1<$qlen) {
	my $output = promer_run($smallref,$smallque,$rend+1,$qend+1,$rlen,$qlen);
	print O $output;
    }

    close(P);
    close(O);
}

sub promer_run {
    my ($smallref,$smallque,$rstart,$qstart,$rend,$qend)=@_;

    my $output="";

    my $rseq=substr($smallref,$rstart-1,$rend-$rstart+1);
    my $qseq=substr($smallque,$qstart-1,$qend-$qstart+1);

    my $reftempfile="reference.temp";
    my $quetempfile="query.temp";

    open(OR,">$reftempfile");
    print OR ">reference\n$rseq";
    close(OR);

    open(OQ,">$quetempfile");
    print OQ ">query\n$qseq";
    close(OQ);
    
    my $command="promer -c $cluster -l $minmatch -p promer.temp.2 reference.temp query.temp";
    print "COMMAND: $command\n";
    my $status=system($command);
    die print STDERR "ERROR: promer exited funny: $?" unless $status ==0;

    my $promertemp2="promer.temp.2.delta";
    open(G,$promertemp2);
    <G>;<G>;<G>;
    while(<G>) {
	my @a=split;
	if($#a>1) {
	    if(($a[0]<=$a[1]) && ($a[2]<=$a[3])) { # positive alignment
		$output.=$a[0]+$rstart-1;$output.=" ";
		$output.=$a[1]+$rstart-1;$output.=" ";
		$output.=$a[2]+$qstart-1;$output.=" ";
		$output.=$a[3]+$qstart-1;
		$output.=" ".$a[4]." ".$a[5]." ".$a[6]."\n";

                my $line;
		do {
		    $line=<G>;
		    $output.=$line;
		    chomp($line);
		} while ($line != 0);
	    }
	    else { # ignore alignment
                my $line;
		do {
		    $line=<G>;
		    chomp($line);
		} while($line != 0);
	    }
	}
    }
    close(G);

    unlink <promer.temp.2.*>;
    unlink <reference.temp>;
    unlink <query.temp>;

    return($output);

}

